# ------------------ Define Type Macros ------------------ #
ctypedef int PASAINT
ctypedef double PASAFLOAT
ctypedef int PPINT
ctypedef double PPFLOAT
ctypedef int CGINT
ctypedef double CGFLOAT
ctypedef int NAPINT
ctypedef double NAPFLOAT

cdef extern from "pasa.h":
    # -------- pasa functions -------- #
    void pasa (PASAdata *pasadata) 
    PASAdata *pasa_setup ()
    void pasa_terminate (PASAdata **Data)
    void pasa_print_parm (PASAdata *Data)
    void pasa_print_status (int status, PASAstats *Stats)

    # -------- pasa structs -------- #
    ctypedef struct PASAdata:
        # -------- pasa input data -------- #
        PASAFLOAT         *x # size ncol, points to a guess for the solution
                             # when pasa is run; at completion of pasa, points
                             # to the computed solution 
        PASAFLOAT   *plambda # If not NULL and pasaparm->use_lambda = T,
                             #   then lambda points to an array of size nrow
                             #   (allocated by the user) that contains a
                             #   guess for the multiplier. If not NULL,
                             #   then at completion, lambda points to
                             #   the computed multiplier.
        PASAparms     *Parms # NULL => use default parameters, otherwise
                             # contains up to 4 non-NULL structures pasa, cg,
                             # pproj, and napheap with parameters
        PASAstats     *Stats # NULL => do not return statistics structure.
                             #   Otherwise, up to 4 non-NULL structures are
                             #   returned with statistics for the codes used.
        PASAINT         nrow # number of rows in A
        PASAINT         ncol # number of components in x and number of cols in
                             # A if it exists
        # If A exists, then its nonzero elements are stored in sparse matrix
        # format corresponding to two integer arrays and one real array. When
        # A does not exist, these arrays should be NULL 
        PASAINT          *Ap # size ncol+1, column pointers
        PASAINT          *Ai # size Ap [ncol], row indices for A, increasing
                             # order in each column
        PASAFLOAT        *Ax # size Ap [ncol], numerical entries of A
        PASAFLOAT        *bl # size nrow, lower bounds for inequalities
                             # NULL => use -infinity
        PASAFLOAT        *bu # size nrow, upper bounds for inequalities
                             # NULL => use +infinity
        PASAFLOAT        *lo # size ncol, lower bounds for x
                             # NULL => use -infinity
        PASAFLOAT        *hi # size ncol, upper bounds for x
                             # NULL => use +infinity
        PASAFLOAT         *y # size ncol, project y onto polyhedron in PPROJ
                             # NULL => not polyhedral projection problem
        PASAFLOAT         *c # size ncol, linear term if objective quadratic
                             # NULL => objective not quadratic
        PASAFLOAT         *a # size ncol, napsack constraint *bl <= a'x <= *bu
                             # NULL => not napsack problem
        PASAFLOAT         *d # size ncol, diagonal of Hessain in NAPHEAP
                             # NULL => no diagonal Hessian in NAPHEAP
        PASAFLOAT     *xWork # NULL => pasa should allocate real work space.
                             # Otherwise, see allocation section of pasa to
                             # determine memory requirements.
        PASAINT       *iWork # NULL  => pasa should allocate integer work space.
                             # Otherwise, see allocation section of pasa to
                             # determine memory requirements.
        CGdata       *cgdata # input data for CG_DESCENT
        PPdata       *ppdata # input data structure for PPROJ
        NAPdata     *napdata # input data for NAPHEAP
        # for a quadratic objective, hprod evaluates Hessian times vector,
        # NULL if objective not quadratic
        void   (*hprod) (double *, double *, int *, int, int)
        # for a quadratic objective in unconstrained problem, cg_hprod evaluates 
        # Hessian times vector, NULL if problem constrained
        void   (*cg_hprod) (double *, double *, int)
        # for a general nonlinear function, value (f, x, n) is function value,
        # NULL if objective is quadratic
        void   (*value) (double *, double *, int)
        # for a general nonlinear function, grad (g, x, n) is function gradient,
        # NULL if objective is quadratic
        void    (*grad) (double *, double *, int) # grad (g, x, n)
        # for a general nonlinear function, valgrad (f, g, x, n) gives both
        # function value f and gradient g at x (size n), argument can be NULL
        void (*valgrad) (double *, double *, double *, int)
        # Hessian evaluation routine, can be NULL, not currently used
        #void    (*hess) (PASAhess *)
        # Hessian evaluation routine for cg, can be NULL, not currently used
        #void    (*cghess) (CGhess *)
        # the following are used internally when the code allocates memory
        PASAFLOAT *x_created ;      # pointer to memory created for x
        PASAFLOAT *lambda_created ; # pointer to memory created for lambda

    ctypedef struct PASAparms:
        PASAparm   *pasa # parameters for PASA, NULL => use default
        CGparm       *cg # parameters for CG_DESCENT, NULL => use default
        PPparm    *pproj # parameters for PPROJ, NULL => use default
        NAPparm *napheap # parameters for NAPHEAP, NULL => use default

    ctypedef struct PASAstats:
        PASAstat   *pasa # statistics for PASA
        CGstat       *cg # statistics for CG_DESCENT
        PPstat    *pproj # statistics for PPROJ
        NAPstat *napheap # statistics for NAPHEAP
        int     use_pasa # T => pasa was used
        int       use_cg # T => cg was used
        int    use_pproj # T => pproj was used
        int  use_napheap # T => napheap was used

    ctypedef struct PASAparm:
        # PASA parameters; descriptions in PASA/Include/pasa.h
        int UNC ;     
        int BNC ;     
        int LP  ;     
        int QP  ;     
        int NL  ;     
        int NAPSACK ; 
        int PROJ ;    
        PASAFLOAT grad_tol ;
        int PrintStatus ;
        int PrintStat ;
        int PrintParm ;
        int PrintLevel ;
        int GradProjOnly ;
        int use_activeGP ;
        int use_napheap ;
        int use_hessian ;
        int loExists ;
        int hiExists ;
        int updateorder ; 
        PASAFLOAT epsilon ; 
        PASAFLOAT cerr_decay ;
        PASAFLOAT EpsilonGrow ;
        PASAFLOAT EpsilonDecay ;
        PASAFLOAT fadjust ;
        int use_lambda ;
        int pproj_start_guess ;
        int use_penalty ;
        PASAFLOAT penalty ;
        int          debug ;
        PASAFLOAT debugtol ;
        PASAFLOAT switchfactor; 
        PASAFLOAT switchdecay ; 
        int terminate_agp ;
        PASAINT testit ; 
        PASAFLOAT GPtol ; 
        PASAINT gpmaxit ; 
        PASAFLOAT restart_fac ;
        int            L ;
        int            M ; 
        int            P ;
        PASAFLOAT gamma1 ;
        PASAFLOAT gamma2 ;
        PASAFLOAT gamma3 ;
        PASAFLOAT        lambda0 ;
        PASAFLOAT  lambda0Factor ;
        PASAFLOAT            bbk ;
        PASAFLOAT       bbexpand ;
        PASAFLOAT bbSwitchFactor ;
        int MaximumCycle ;
        int NominalCycle ;
        int approxstep ;
        PASAFLOAT ArmijoSwitchFactor ;
        int  PertRule ;
        PASAFLOAT pert_eps ;
        int neps ;
        PASAFLOAT Armijo_delta ;
        PASAFLOAT Wolfe_delta ;
        PASAFLOAT Wolfe_sigma ;
        int maxsteps ;
        PASAFLOAT stepdecay ;
        PASAFLOAT safe0 ;
        PASAFLOAT safe1 ;
        PASAFLOAT infdecay ;
        PASAFLOAT infdecay_rate ;
        int     ninf_tries ;

    ctypedef struct PASAstat:
        int         status ; #  status at termination 
        int       maxsteps ; #  max number of attempts to satisfy the grad proj
                             #  line search condition 
        PASAFLOAT    lobad ; #  invalid lower bound for a variable 
        PASAFLOAT    hibad ; #  invalid upper bound for a variable 
        PASAINT       ibad ; #  index of the bad bound 
        PASAFLOAT grad_tol ; #  KKT error tolerance 
        int     ninf_tries ; #  number of tries to find finite objective value 
        PASAINT    gpmaxit ; #  max iteration in gradient projection algorithm 
        PASAFLOAT        f ; #  function value 
        PASAFLOAT      err ; #  || P (x - g) - x || 
        PASAINT       mcnf ; #  function evaluations in main code 
        PASAINT       mcng ; #  gradient evaluations in main code 
        PASAINT       gpit ; #  number of iterations in grad_proj 
        PASAINT       gpnf ; #  function evaluations in grad_proj 
        PASAINT       gpng ; #  gradient evaluations in grad_proj 
        PASAINT      agpit ; #  number of iterations in active set grad_proj 
        PASAINT      agpnf ; #  function evaluations in active set grad_proj 
        PASAINT      agpng ; #  gradient evaluations in active set grad_proj 
        PASAINT   nproject ; #  number of projections performed 

cdef extern from "pproj.h":
    # -------- pproj functions -------- #
    void pproj_print_parm (PPdata *Data)

    # -------- pproj structs -------- #
    ctypedef struct PPdata:
        pass

    ctypedef struct PPparm:
        # PPROJ parameters; descriptions in PPROJ/Include/pproj.h
        int    PrintStatus ; 
        int      PrintStat ; 
        int     PrintLevel ; 
        int      PrintParm ; 
        int use_prior_data ; 
        int    return_data ; 
        int       loExists ; 
        int       hiExists ; 
        int      getfactor ; 
        int          debug ; 
        PPFLOAT   checktol ; 
        int    start_guess ; 
        int        permute ; 
        PPFLOAT     phase1 ; 
        int        cholmod ; 
        int     multilevel ; 
        int stop_condition ; 
        PPFLOAT      sigma ; 
        PPFLOAT     Asigma ; 
        int     ScaleSigma ; 
        PPFLOAT sigma_decay ; 
        int          nprox ; 
        PPFLOAT armijo_grow; 
        int        narmijo ; 
        int            mem ; 
        int        nsparsa ; 
        PPFLOAT      gamma ; 
        PPFLOAT        tau ; 
        PPFLOAT       beta ; 
        PPFLOAT grad_decay ; 
        PPFLOAT gamma_decay; 
        int use_coor_ascent; 
        PPFLOAT   coorcost ; 
        int      use_ssor0 ; 
        int      use_ssor1 ; 
        int     use_sparsa ; 
        int    use_startup ; 
        PPFLOAT  ssordecay ; 
        PPFLOAT   ssorcost ; 
        int        ssormem ; 
        PPINT   ssormaxits ; 
        PPFLOAT  cutfactor ; 
        PPFLOAT    tolssor ; 
        PPFLOAT    tolprox ; 
        PPFLOAT tolrefactor; 
        int badFactorCutoff; 
        int             LP ; 
        PPFLOAT  LinFactor ; 
        PPFLOAT LinGrad_tol; 

    ctypedef struct PPstat:
        int             status ; #  status of the run 
        int         parm_nprox ; #  max number of proximal updates 
        int            cholmod ; #  TRUE => cholmod is employed 
        PPFLOAT          lobad ; #  invalid lower bound for a linear inequality 
        PPFLOAT          hibad ; #  invalid upper bound for a linear inequality 
        PPINT             ibad ; #  index of the bad bound 
        PPFLOAT       grad_tol ; #  relative tolerance for dual function gradient 
        PPINT       ssormaxits ; #  upper bound on number of ssor iterations 
        PPFLOAT        errdual ; #  actual error in the dual function gradient where
                                 #  the error measure is based on the stop condition
                                 #  0 => ||grad L (lambda)||_sup / absAx_sup
                                 #       where absAx_sup = max_i sum_j |a_{ij}x_j|
                                 #       x achieves min in dual function
                                 #  1 => ||grad L (lambda)||_sup
                                 #  2 => ||grad L (lambda)||_sup /(absAx_sup + ymax)
                                 #       where ymax is the sup-norm of the
                                 #       projection point y 
        PPINT             nrow ; #  number of rows in A 
        int           *updowns ; #  number of updates and downdates of each size 
        int       size_updowns ; #  dimension of updowns array 
        int            *solves ; #  size: maxdepth+1, # of solves by level  
        int           maxdepth ; #  number of levels in the partition tree 
        int               blks ; #  number of blocks in multilevel partition of A
                                 #  each separator is also counted as a block 
        int             nchols ; #  number of Cholesky factorizations 
        int              nprox ; #  number of proximal updates 
        PPINT       phase1_its ; #  number of iterations in phase 1 
        PPINT  coor_ascent_its ; #  number of coordinate ascent iterations 
        PPINT        ssor0_its ; #  number of ssor0 iterations 
        PPINT        ssor1_its ; #  number of ssor1 iterations 
        PPINT       sparsa_its ; #  number of SpaRSA iterations 
        PPINT            coldn ; #  number of rank 1 downdates to Cholesky factor 
        PPINT            colup ; #  number of rank 1 updates to Cholesky factor 
        PPINT            rowdn ; #  number of rows dropped from Cholesky factor 
        PPINT            rowup ; #  number of rows added to Cholesky factor 
        PPINT coor_ascent_free ; #  number of variables freed in coordinate ascent
        PPINT coor_ascent_drop ; #  number of rows dropped in coordinate ascent 
        PPINT       ssor0_free ; #  number of variables freed in ssor0 
        PPINT       ssor0_drop ; #  number of rows dropped in ssor0 
        PPINT       ssor1_free ; #  number of variables freed in ssor1 
        PPINT       ssor1_drop ; #  number of rows dropped in ssor1 
        PPINT       sparsa_col ; #  number of bound constraint changes in SpaRSA 
        PPINT       sparsa_row ; #  number of changes in row constraints in SpaRSA
        PPINT sparsa_step_fail ; #  number of failures of Armijo step in SpaRSA 
        PPINT             lnnz ; #  number of nonzeros in final Cholesky factor 
    
        #  timing 
        PPFLOAT    partition ; #  compute reordering of rows of A 
        PPFLOAT   initialize ; #  initwork and initlevels, includes partition 
        PPFLOAT       phase1 ; #  phase1 
        PPFLOAT       sparsa ; #  sparsa 
        PPFLOAT  coor_ascent ; #  coor_ascent 
        PPFLOAT        ssor0 ; #  ssor0 
        PPFLOAT        ssor1 ; #  ssor1 
        PPFLOAT         dasa ; #  dasa (includes coor_ascent, ssor0, and ssor1) 
        PPFLOAT    dasa_line ; #  dasa line search 
        PPFLOAT     checkerr ; #  check_error 
        PPFLOAT  prox_update ; #  prox_update 
        PPFLOAT       invert ; #  invert permutation of rows and columns 
        PPFLOAT       modrow ; #  modrow (update L by adding or deleting rows) 
        PPFLOAT       modcol ; #  modcol (rank 1 column updates of L) 
        PPFLOAT         chol ; #  cholmod_analyze, cholmod_factorize 
        PPFLOAT      cholinc ; #  incremental cholmod_rowfac 
        PPFLOAT     dltsolve ; #  dltsolve (back solve) 
        PPFLOAT       lsolve ; #  lsolve (forward solve) 

cdef extern from "cg_descent.h":
    # -------- cg functions -------- #
    void cg_print_parm (CGdata *Data)

    # -------- cg structs -------- #
    ctypedef struct CGdata:
        pass

    ctypedef struct CGparm:
        #  T => print status of run
        #  F => do not print status of run 
        int PrintStatus ;
    
        #  T => print pasa statistics
        #  F => do not print statistics 
        int PrintStat ;
    
        #  T => print parameter values 
        #  F => do not print parameter values 
        int PrintParm ;
    
        #  Level 0  = no printing, ... , Level 2 = maximum printing 
        int PrintLevel ;
    
        #  T => objective function is a quadratic
        #  F => objective function not necessarily quadratic 
        #int QuadCost ;
    
        #  an adjustment added to a quadratic cost objective when it is evaluated 
        CGFLOAT fadjust ;
    
        #  replace the Hessian Q of a quadratic by Q + QPshift 
        CGFLOAT QPshift ;
    
        #  stopping tolerance 
        CGFLOAT grad_tol ;

        # CG_DESCENT no longer includes the stopping criterion

        #    ||proj_grad||_infty <= grad_tol*(1 + |f_k|).

        # If the optimization problem is unconstrained, then the stopping
        # criterion is

        #    ||proj_grad||_infty <= testtol

        # where testtol = max(grad_tol,initial ||grad||_infty*StopFact) and
        # the default value of StopFact is zero. If the optimization problem
        # contains constraints, testtol is the pasa stopping criterion
        # switchfactor*global_error. This value for testtol is the PASA
        # criterion to stop solving the unconstrained problem and return
        # to the gradient project algorithm. */
        CGFLOAT StopFac ;
    
        #  T => check that f_k+1 - f_k <= debugtol*C_k
        #  F => no checking of function values 
        int debug ;
        CGFLOAT debugtol ;
    
        #  if step is nonzero, it is the initial step of the initial line search 
        CGFLOAT step ;
    
        #  0 => use cg_descent
        #  1 => use L-BFGS
        #  2 => use L-BFGS when LBFGSmemory >= n, use cg_descent when memory < n
        #  3 => use L-BFGS when LBFGSmemory >= n, use limited memory CG otherwise 
        int LBFGS ;
    
        #  if LBFGS is used, then LBFGSmemory is the number of vectors in memory 
        int LBFGSmemory ;
    
        #  abort cg after maxit iterations 
        CGINT maxit ;
    
        #  conjugate gradient method restarts after (n*restart_fac) iterations 
        CGFLOAT restart_fac ;
    
        #  factor in [0, 1] used to compute average cost magnitude C_k as follows:
        #  Q_k = 1 + (Qdecay)Q_k-1, Q_0 = 0,  C_k = C_k-1 + (|f_k| - C_k-1)/Q_k 
        CGFLOAT Qdecay ;
    
        #  terminate after nslow iterations without strict improvement in
        #  either function value or gradient 
        int nslow ;
    
        #  factor by which eps grows when line search fails during contraction 
        CGFLOAT egrow ;
    
        #  T => attempt quadratic interpolation in line search when
        #           |f_k+1 - f_k|/f_k <= QuadCutoff
        #  F => no quadratic interpolation step 
        int    QuadStep ;
        CGFLOAT QuadCutOff ;
    
        #  maximum factor by which a quad step can reduce the step size 
        CGFLOAT QuadSafe ;
    
        CGFLOAT psi_lo ; #  in performing a QuadStep, we evaluate at point
                         #  betweeen [psi_lo, psi_hi]*psi2*previous step 
        CGFLOAT psi_hi ;
        CGFLOAT   psi1 ; #  for approximate quadratic, use gradient at
                         #  psi1*psi2*previous step for initial stepsize 
    
        CGFLOAT   qeps ; #  parameter in cost error for quadratic restart
                         #  criterion 
        CGFLOAT  qrule ; #  parameter used to decide if cost is quadratic 
        int   qrestart ; #  number of iterations the function should be
                         #  nearly quadratic before a restart 
    
        #  T => when possible, use a cubic step in the line search 
        int UseCubic ;
    
        #  use cubic step when |f_k+1 - f_k|/|f_k| > CubicCutOff 
        CGFLOAT CubicCutOff ;
    
        #  |f| < SmallCost*starting cost => skip QuadStep and set PertRule = FALSE
        CGFLOAT SmallCost ;
    
        #  maximum factor secant step increases stepsize in expansion phase 
        CGFLOAT ExpandSafe ;
    
        #  factor by which secant step is amplified during expansion phase
        #  where minimizer is bracketed 
        CGFLOAT SecantAmp ;
    
        #  factor by which rho grows during expansion phase where minimizer is
        #  bracketed 
        CGFLOAT RhoGrow ;
    
        #  If approxstep is TRUE, use approximate Wolfe line search.
        #  If approxstep is FALSE, then use ordinary line search and
        #  switch to the approximate step when |f - fnew| < ApproxSwitchFactor*|fR|,
        #  where fR is an estimate of the function size.  In the conjugate
        #  gradient code, an averaging technique is used to estimate function
        #  size. 
        int approxstep ;
        CGFLOAT ApproxSwitchFactor ;
    
        #  As the cg_descent converges, the function values typically
        #  approach a constant value. When this happens, the cubic interpolation
        #  step in the line search loses its accuracy and it is better to
        #  use a secant step based on the derivative of the objective function.
        #  The cost has converged when the relative change in the objective
        #  function <= CostConverge 
        CGFLOAT CostConverge ;
    
        CGFLOAT FuncGradSwitchFactor ;
    
        #  When performing an approximate Wolfe line search, we require
        #  that the new function value <= perturbation of the prior
        #  function value where the perturbation tries to take into
        #  rounding errors associated with the function value.
        #  There are two different perturbations:
        #  PertRule = 1 => fpert is f + Parm->pert_eps*|f| (relative change)
        #  PertRule = 0 => fpert is f + Parm->pert_eps     (absolute change) 
        int PertRule ;
        CGFLOAT pert_eps ;
    
        #  Maximum number of contractions in cg_contract. If it cannot find a
        #  step that either satisfies the Wolfe conditions or which has derivative
        #  >= 0 within ncontract attempts, then it is felt that fpert is too
        #  small and it will be increased so that the current function value
        #  is less than fpert. To increase fpert, we increase the value of
        #  pert_eps which is used to compute fpert. 
        int ncontract ;
    
        #  When pert_eps is increased, we multiply the new value by the growth
        #  factor eps_grow to ensure a healthy growth in pert_eps. 
        CGFLOAT eps_grow ;
    
        #  Maximum number of times that pert_eps is recomputed before a line
        #  search error is declared. 
        int neps ;
    
        CGFLOAT    cgdelta ; #  Wolfe line search parameter 
        CGFLOAT    cgsigma ; #  Wolfe line search parameter 
        int       maxsteps ; #  max number of tries to find acceptable step 
        CGFLOAT  stepdecay ; #  decay factor for bracket interval width 
        CGFLOAT        rho ; #  growth factor when searching for initial
                             #  bracketing interval 
        CGFLOAT       psi0 ; #  factor used in starting guess for iteration 1 
        CGFLOAT       psi2 ; #  when starting a new cg iteration, our initial
                             #  guess for the line search stepsize is
                             #  psi2*previous step 
        CGFLOAT  BetaLower ; #  parameter connected with lower bound for beta 
        CGFLOAT      theta ; #  parameter describing the cg_descent family 
        int   AdaptiveTheta ; #  T => choose theta adaptively, F => use theta 
    
        #  When an infinite or nan objective value is encountered, the
        #  stepsize is reduced in an effort to find a finite objective
        #  value. infdecay is the initial decay factor that is used when an
        #  infinite or nan objective value is encountered, ninf_tries is
        #  the number of attempts we make to find a finite objective value,
        #  and infdecay_rate is a factor by which infdecay is multiplied
        #  after each attampt to find a finite objective value. 
        CGFLOAT cg_infdecay ;
        CGFLOAT cg_infdecay_rate ;
        int  cg_ninf_tries ;
    
    #  ============ LIMITED MEMORY CG PARAMETERS ================================ 
        #  SubCheck and SubSkip control the frequency with which the subspace
        #  condition is checked. It it checked for SubCheck*mem iterations and
        #  if it is not activated, then it is skipped for Subskip*mem iterations
        #  and Subskip is doubled. Whenever the subspace condition is satisfied,
        #  SubSkip is returned to its original value. 
        int SubCheck ;
        int SubSkip ;
    
        #  when relative distance from current gradient to subspace <= eta0,
        #  enter subspace if subspace dimension = mem (eta0 = 0 means gradient
        #  inside subspace) 
    
        CGFLOAT   eta0 ; #  corresponds to eta0*eta0 in the paper 
    
        #  when relative distance from current gradient to subspace >= eta1,
        #  leave subspace (eta1 = 1 means gradient orthogonal to subspace) 
        CGFLOAT   eta1 ; #  corresponds to eta1*eta1 in the paper 
    
        #  when relative distance from current gradient to subspace <= eta2,
        #  always enter subspace (invariant space) 
        CGFLOAT   eta2 ;

    ctypedef struct CGstat:
        int         status ; #  returned status from cg_descent 
        CGFLOAT          f ; #  function value at solution 
        CGFLOAT        err ; #  sup norm of the gradient 
        CGFLOAT   grad_tol ; #  error tolerance 
        CGFLOAT      gnorm ; #  max abs component of gradient 
        CGFLOAT        tol ; #  computing tolerance: gnorm <= tol 
        CGINT        maxit ; #  maximum number of iterations 
        int  cg_ninf_tries ; #  number of tries to find finite objective value 
        CGFLOAT       oldf ; #  old function value when debugger fails 
        CGFLOAT       newf ; #  new function value when debugger fails 
        int       maxsteps ; #  max number of attempts in the line search 
        int        NegDiag ; #  T => negative diagonal encountered in QP factor 
        CGINT         iter ; #  number of iterations in cg 
        CGINT        nfunc ; #  function evaluations in cg 
        CGINT        ngrad ; #  gradient evaluations in cg 
        int        IterSub ; #  number subspace iterations in limited memory cg 
        int         NumSub ; #  number of subspaces in limited memory cg 

cdef extern from "napheap.h":
    # -------- napheap functions -------- #
    void napheap_print_parm (NAPdata *Data)

    # -------- napheap structs -------- #
    ctypedef struct NAPdata:
        pass

    ctypedef struct NAPparm:
        #  T => print status of run 
        int PrintStatus ;
    
        #  T => print statistics of the run 
        int PrintStat ;
    
        #  T => print the parameters 
        int PrintParm ;
    
        #  T => the prior_data input argument of napheap contains
        #       problem data from a previous run 
        int use_prior_data ;
    
        #  T => return the data structure for the current solution in the
        #       napcom argument of napheap (for use in another run with a
        #       different y value) 
        int return_data ;
    
        #  F => treat input argument lo as -infinity 
        int loExists ;
    
        #  F => treat input argument hi as +infinity 
        int hiExists ;
    
        #  F => no linear constraints bl <= a'x <= bu 
        int Aexists ;
    
        #  T => diagonal of objective Hessian is all positive, and the
        #       MATLAB expression all(d>0) is assumed to be true.
        #  F => there could be zeros on the diagonal of the Hessian 
        int d_is_pos ;
    
        #  T => diagonal of objective Hessian is zero.
        #       Note: The d argument of napheap is ignored. It could be NULL
        #  F => there could be nonzeros on the diagonal of the Hessian 
        int d_is_zero ;
    
        #  T => diagonal of objective Hessian is identically 1.
        #       Note: The d argument of napheap is ignored. It could be NULL 
        int d_is_one ;
    
        #  T => use input lambda as a starting guess for multiplier
        #  F => ignore input lambda (final lambda is returned in lambda argument) 
        #int use_lambda ;
    
        #  Upper bound on the number of Newton or variable fixing iterations.
        #  After performing these K iterations, the code switches to the break
        #  point search. The Newton algorithm also switches to the breakpoint
        #  search if two iterates are generated on opposite sides of the root. 
        int K ;
    
        #  T => use the Newton to update lambda, switch to the break point
        #  search either at iteration K or at the first time that two
        #  Newton iterates lie on opposite side of the root (whichever
        #  happens first)
        #  F => use K iterations of the variable fixing algorithm before
        #  switching to a breakpoint search 
        int newton ;
    
        #  An ordinary Newton iteration often converges monotonically to the
        #  root. Hence, the Newton iterates are scaled by a factor greater than 1
        #  in an effort to produce an iterate on the opposite side of the root.
        #  When such an iterate is generated, the algorithm switches to a
        #  breakpoint search. The scaled iteration is given by
        #               
        #  lambda_new = lambda_old - newton_scale*L'(lambda_old)/L''(lambda_old) 
        #               
        NAPFLOAT newton_scale ;
    
        #  To combat the effect of rounding errors, recompute a2sum whenever
        #  a2sum <= decay*a2max, the maximum a2sum encountered 
        double decay ;
    
        #  T => perform an iterative refinement step to improve solution accurary
        #  F => no refinement 
        int refine ;
    
        #  The solution is refined if |a'x - b| > err (only if refine is true) 
        NAPFLOAT err ;
    
        #  Check the input parameters for dj < 0 or hij < loj or Bhi < Blo 
        int check ;

    ctypedef struct NAPstat:
        NAPINT     nkf ;   #  number of known free variables 
        NAPINT   nfree ;   #  number of free variables in initial heap 
        NAPINT  nbound ;   #  number of bound variables in initial heap 
        NAPINT   nbrks ;   #  number break points to reach initial solution 
        NAPINT nrefine ;   #  number of break points during refinement 
        NAPINT nvarfix ;   #  number of variable fixing iterations 
        NAPINT nnewton ;   #  number of Newton iterations performed 
        NAPINT nsecant ;   #  number of secant steps performed 
    
        #  for error reporting 
        int     status ;   #  returned status from napheap 
        NAPINT  kerror ;   #  invalid n, or index where error found 
        NAPFLOAT lobad ;   #  bad value of lo, if any 
        NAPFLOAT hibad ;   #  bad value of hi, if any 
        NAPFLOAT  dbad ;   #  bad value of d, if any 



  ############################################################################
  #                                                                          #
  #  Demo problem: Solve a problem from the CUTEst test set.                 #
  #                                                                          #
  ############################################################################

# ----------------------------- Import libraries ----------------------------- #
import SuiteOPT
import pycutest
import numpy as np
import scipy

# -------------------- Functions to preprocess CUTEst data ------------------- #
def preprocess_constraints (cute_problem, problem):
  # Get lower bound on variable from pycutest, lo <= x
  lo = cute_problem.bl
  # Set extreme negative values to -infinity
  lo[lo == -1e+20] = np.NINF
  # Check if all values are not infinite
  if np.mean(np.isinf(lo)) != 1:
    # Store lo in problem
    problem.lo = lo
  # Option to save to file
  #np.save("lo.npy", lo)

  # Get upper bound on variable from pycutest, x <= hi
  hi = cute_problem.bu
  # Set extreme positive values to infinity
  hi[hi == 1e+20] = np.Inf
  # Check if all values are not infinite
  if np.mean(np.isinf(hi)) != 1:
    # Store hi in problem
    problem.hi = hi
  # Option to save to file
  #np.save("hi.npy", hi)

  # Get constraint matrix A for polyhedral problem by evaluating at any point
  #gs, A = cute_problem.slagjac(cute_problem.x0)
  gs, A = cute_problem.slagjac(np.zeros(cute_problem.n))
  # Check if problem has linear constraint matrix A
  if A is None:
    # No linear constrints; exit program
    return
  # Convert constraint matrix to csc format for SuiteOPT
  A = A.tocsc()
  # Store A in problem
  problem.A = A
  # Option to save to file
  #scipy.sparse.save_npz("A.npz", A)

  # Get lower bound on linear constraint from pycutest, cl <= Ax
  cl = cute_problem.cl
  # Get upper bound on linear constraint from pycutest, Ax <= cu
  cu = cute_problem.cu
  # Set extreme negative values to -infinity
  cl[cl == -1e+20] = np.NINF
  # Set extreme positive values to infinity
  cu[cu == 1e+20] = np.Inf
  # Compute offset for linear constraints
  offset = cute_problem.cons(np.zeros(cute_problem.n))
  # Subtract offset vector from cl and cu and store in bl and bu
  bl = np.zeros(cute_problem.m)
  bu = np.zeros(cute_problem.m)
  for i in range(cute_problem.m):
    bl[i] = cl[i] - offset[i]
    bu[i] = cu[i] - offset[i]
  # Check if all values are not infinite
  if np.mean(np.isinf(bl)) != 1:
    # Store bl in problem
    problem.bl = bl
  # Check if all values are not infinite
  if np.mean(np.isinf(bu)) != 1:
    # Store bu in problem
    problem.bu = bu
  # Option to save to file
  #np.save("bl.npy", bl)
  #np.save("bu.npy", bu)

  # Exit program
  return

# ------------------------- Function to solve problem ------------------------ #
def solve_problem(problem_name):
  # ------------------------ Build CUTEst Test Problem ----------------------- #
  # Clear cached problem in case there is an error with it
  pycutest.clear_cache(problem_name)
  # Build requested problem
  cute_problem = pycutest.import_problem(problem_name, efirst = True, lfirst = True, nvfirst = False)

  # ---------------------- Initialize problem parameters --------------------- #
  parm = dict(PrintStat=1, PrintLevel=0, PrintParm=1)
  
  # ------------------------- Initialize problem class ----------------------- #
  problem = SuiteOPT.problem(cute_problem.n, cute_problem.m)
  
  # ------------ Fill in problem parameters based on problem type ------------ #
  problem.x  = cute_problem.x0  # Initial guess for solution
  # Polyhedral constraints, lo <= x <= hi and bl <= Ax <= bu
  preprocess_constraints (cute_problem, problem) 
  # Hessian of cost function
  H = cute_problem.ihess(np.zeros(problem.ncol))
  H = scipy.sparse.csc_matrix(H)
  # Option to save to file
  #scipy.sparse.save_npz("H.npz", H)
  # Extract sparse matrix components for hprod
  Hx = H.data    # Hessian matrix values
  Hp = H.indptr  # Hessian matrix column pointers
  Hi = H.indices # Hessian matrix row indices
  
  # ---------------------- Wrappers for cutest functions --------------------- #
  def objective(x):
    return cute_problem.obj(x)
  
  def gradient(x):
    f, g = cute_problem.obj(x, gradient=True)
    return g
  
  def valgrad(x):
    return cute_problem.obj(x, gradient=True)
  
  def hprod(d,ifree):
    # Initialize m to length of vector d
    m = len(d)
    # Initialize n to length of vector ifree
    n = len(ifree)
    # Initialize output array to zeros
    Hd = np.zeros(m)
    # Check if ifree array is nonempty
    if len(ifree) != 0:
      for j in range(n):
        t = d[j]
        if t != 0:
          k = ifree[j]
          q = Hp[k+1]
          for p in range(Hp[k], q):
            Hd[ Hi[p] ] += t * Hx[p]
    else:
      for j in range(n):
        t = d[j]
        if t != 0:
          q = Hp[j+1]
          for p in range(Hp[j], q):
            Hd[ Hi[p] ] += t * Hx[p]
    return Hd

  def cg_hprod(x):
    return H.dot(x)

  # ------------ Fill in problem functions based on problem type ------------- #
  # First check for constraint types that PASA cannot handle
  if problem_name in (pycutest.find_problems(constraints='Q')+
                      pycutest.find_problems(constraints='O')):
    # Problem has constraints that are nonlinear
    print("Warning: Given problem %s has nonlinear constraints\n" %problem_name)
    #print("EXITING PROGRAM\n\n") 
    #return
    # Set objective, gradient, and objgrad
    problem.objective = objective
    problem.gradient = gradient
    problem.objgrad = valgrad
  
  # Now check for specific problem types
  # Unconstrained problems
  if problem_name in pycutest.find_problems(constraints='U'):
    # Given problem is unconstrained
    problem.lo = None
    problem.hi = None
    if problem_name in pycutest.find_problems(objective='L'):
      # --- Linear programs --- #
      # Set c, lo, hi, bl, bu, and A
      f, g = cute_problem.obj(problem.x, gradient=True)
      # Linear term in cost function
      problem.c = g  
      # Option to save to file
      #np.save("c_dfl.npy", problem.c)
  
    elif problem_name in pycutest.find_problems(objective='Q'):
      # --- Quadratic programs --- #
      # Set c, hprod, lo, hi, bl, bu, and A
      f, g = cute_problem.obj(problem.x, gradient=True)
      # Linear term in cost function
      problem.c = g  
      # Hprod
      problem.cg_hprod = cg_hprod
    else:
      # Set objective, gradient, and objgrad
      problem.objective = objective
      problem.gradient = gradient
      problem.objgrad = valgrad
  elif problem_name in pycutest.find_problems(objective='L'):
    # --- Linear programs --- #
    # Set c, lo, hi, bl, bu, and A
    f, g = cute_problem.obj(problem.x, gradient=True)
    # Linear term in cost function
    problem.c = g  
    # Option to save to file
    #np.save("c_dfl.npy", problem.c)
  elif problem_name in pycutest.find_problems(objective='Q'):
    # --- Quadratic programs --- #
    # Set c and hprod
    f, g = cute_problem.obj(problem.x, gradient=True)
    # Linear term in cost function
    problem.c = g  
    #np.save("c.npy", problem.c)
    # Hprod
    problem.hprod = hprod
  else:
    # --- Nonlinear programs --- #
    # Set objective, gradient, and objgrad
    problem.objective = objective
    problem.gradient = gradient
    problem.objgrad = valgrad
  
  # Fill in problem parameters (bypass any parameters absent from your problem)
  problem.parm = parm  # custom parameter values (optional)
  
  # ---------------------------- Print Problem Name -------------------------- #
  print("Problem name: %s\n" %(cute_problem.name))
  
  # ---------------------------- Print parameters ---------------------------- #
  #SuiteOPT.print_parm("all")

  # ------------------------------- Call solver ------------------------------ #
  newx = SuiteOPT.solve(problem)
  
  # ----------------------------- Print solution ----------------------------- #
  problem.print_stats()
  
  if newx is not None:
      print ("SuiteOPT solution: %s" %str(newx))

  #g = problem.gradient(np.zeros(problem.ncol))
  #print("g(0) = ", g)
  #g = problem.gradient(np.ones(problem.ncol))
  #print("g(1) = ", g)
  #g = problem.gradient(np.arange(problem.ncol))
  #print("g([n]) = ", g)

  #print("bl = ", problem.bl)
  #print("bu = ", problem.bu)
  #print("lo = ", problem.lo)
  #print("hi = ", problem.hi)

#  # ----------------- Compare problem data to C problem data ----------------- #
#  #lo.sort()
#  lo_c = np.genfromtxt('lo_c.csv', delimiter=',')
#  #lo_c.sort()
#  lo_c[lo_c == -1e+20] = np.NINF
#  #print(lo_c)
#  #print(lo)
#  #print(lo == lo_c)
#  #print(lo - lo_c)
#  print(np.mean(problem.lo != lo_c)*len(lo_c))
#
#  #hi.sort()
#  hi_c = np.genfromtxt('hi_c.csv', delimiter=',')
#  #hi_c.sort()
#  hi_c[hi_c == -1e+20] = np.NINF
#  #print(hi_c)
#  #print(hi)
#  #print(hi == hi_c)
#  #print(hi - hi_c)
#  print(np.mean(problem.hi != hi_c)*len(hi_c))
#
#  #bl.sort()
#  bl_c = np.genfromtxt('bl_c.csv', delimiter=',')
#  #bl_c.sort()
#  bl_c[bl_c == -1e+20] = np.NINF
#  #print(bl_c)
#  #print(bl)
#  #print(bl == bl_c)
#  #print(bl - bl_c)
#  print(np.mean(problem.bl != bl_c)*len(bl_c))
#  
#  #bu.sort()
#  bu_c = np.genfromtxt('bu_c.csv', delimiter=',')
#  #bu_c.sort()
#  bu_c[bu_c == 1e+20] = np.Inf
#  #print(bu_c)
#  #print(bu)
#  #print(bu == bu_c)
#  #print(bu - bu_c)
#  print(np.mean(problem.bu != bu_c)*len(bu_c))
#  
#  #offset.sort()
#  offset_c = np.genfromtxt('offset_c.csv', delimiter=',')
#  #offset_c.sort()
#  #offset_c[offset_c == -1e+20] = np.NINF
#  #offset_c[offset_c == 1e+20] = np.Inf
#  #print(offset_c)
#  #print(offset)
#  #print(offset == offset_c)
#  #print(offset - offset_c)
##  print(np.mean(offset != offset_c)*len(offset_c))
#  
#  Ax = problem.A.data
#  #Ax.sort()
#  Ax_c = np.genfromtxt('Ax_c.csv', delimiter=',')
#  #Ax_c.sort()
#  #print(Ax_c)
#  #print(Ax)
#  #print(Ax == Ax_c)
#  #print(Ax - Ax_c)
#  print("Ax = ", problem.A.data)
#  print("Ax_c = ", Ax_c)
#  print(np.mean(problem.A.data != Ax_c)*len(Ax_c))
#
#  Ap_c = np.genfromtxt('Ap_c.csv', delimiter=',', dtype=np.int32)
#  Ai_c = np.genfromtxt('Ai_c.csv', delimiter=',', dtype=np.int32)

  # --------------- Compare problem data to python problem data -------------- #
#  #lo.sort()
#  lo_c = np.genfromtxt('lo_py.csv', delimiter=',')
#  #lo_c.sort()
#  lo_c[lo_c == -1e+20] = np.NINF
#  #print(lo_c)
#  #print(lo)
#  #print(lo == lo_c)
#  #print(lo - lo_c)
#  print(np.mean(problem.lo != lo_c)*len(lo_c))
#
#  #hi.sort()
#  hi_c = np.genfromtxt('hi_py.csv', delimiter=',')
#  #hi_c.sort()
#  hi_c[hi_c == -1e+20] = np.NINF
#  #print(hi_c)
#  #print(hi)
#  #print(hi == hi_c)
#  #print(hi - hi_c)
#  print(np.mean(problem.hi != hi_c)*len(hi_c))
#
#  #bl.sort()
#  bl_c = np.genfromtxt('bl_py.csv', delimiter=',')
#  #bl_c.sort()
#  bl_c[bl_c == -1e+20] = np.NINF
#  #print(bl_c)
#  #print(bl)
#  #print(bl == bl_c)
#  #print(bl - bl_c)
#  print(np.mean(problem.bl != bl_c)*len(bl_c))
#  for i in np.where((problem.bl != bl_c) == True)[0]:
#    print("bl[%i] = %lg\nbl_c[%i] = %lg\n" %(i, problem.bl[i], i, bl_c[i]))
#    print("bl[%i] - bl_c[%i] = %lg\n\n" %(i, i, problem.bl[i] - bl_c[i]))
#  
#  #bu.sort()
#  bu_c = np.genfromtxt('bu_py.csv', delimiter=',')
#  #bu_c.sort()
#  bu_c[bu_c == 1e+20] = np.Inf
#  #print(bu_c)
#  #print(bu)
#  #print(bu == bu_c)
#  #print(bu - bu_c)
#  print(np.mean(problem.bu != bu_c)*len(bu_c))
#  for i in np.where((problem.bu != bu_c) == True)[0]:
#    print("bu[%i] = %lg\nbu_c[%i] = %lg\n" %(i, problem.bu[i], i, bu_c[i]))
#    print("bu[%i] - bu_c[%i] = %lg\n\n" %(i, i, problem.bu[i] - bu_c[i]))
#  
#  #offset.sort()
#  #offset_c = np.genfromtxt('offset_c.csv', delimiter=',')
#  #offset_c.sort()
#  #offset_c[offset_c == -1e+20] = np.NINF
#  #offset_c[offset_c == 1e+20] = np.Inf
#  #print(offset_c)
#  #print(offset)
#  #print(offset == offset_c)
#  #print(offset - offset_c)
#  #print(np.mean(offset != offset_c)*len(offset_c))
#  
#  Ax = problem.A.data
#  #Ax.sort()
#  Ax_c = np.genfromtxt('Ax_py.csv', delimiter=',')
#  #Ax_c.sort()
#  #print(Ax_c)
#  #print(Ax)
#  #print(Ax == Ax_c)
#  #print(Ax - Ax_c)
#  #print("Ax = ", problem.A.data)
#  #print("Ax_c = ", Ax_c)
#  print(np.mean(problem.A.data != Ax_c)*len(Ax_c))
#  for i in np.where((problem.A.data != Ax_c) == True)[0]:
#    print("Ax[%i] = %lg\nAx_c[%i] = %lg\n" %(i, problem.A.data[i], i, Ax_c[i]))
#    print("Ax[%i] - Ax_c[%i] = %lg\n\n" %(i, i, problem.A.data[i] - Ax_c[i]))
#
#  #Ap_c = np.genfromtxt('Ap_c.csv', delimiter=',', dtype=np.int32)
#  #Ai_c = np.genfromtxt('Ai_c.csv', delimiter=',', dtype=np.int32)
#
#  # Check ordering of returned problem information
#  print("Equations before inequalities: ", cute_problem.eq_cons_first)
#  print("Linear before nonlinear constraints: ", cute_problem.linear_cons_first)
#  print("Nonlinear variables first: ", cute_problem.nonlinear_vars_first)

#  problem.lo = lo_c
#  problem.bl = bl_c
#  problem.bu = bu_c
#  problem.A.data = Ax_c
#  problem.A.indptr = Ap_c
#  problem.A.indices = Ai_c
#
#  # -------------------------------- Call solver ------------------------------- #
#  newx = SuiteOPT.solve(problem)
#  
#  # ------------------------------ Print solution ------------------------------ #
#  problem.print_stats()
#  
#  if newx is not None:
#      print ("SuiteOPT solution: %s" %str(newx))
  

# ------------------------ Get problem name from user ------------------------ #
pname = raw_input('Enter problem name: ')
#pname = 'AGG'
#pname = 'LUKVLE16'
solve_problem(pname)



# ----------------------------- Problem Details ------------------------------ #
#  NAME          DFL001                                                            
#  An LP, contributed by Marc Meketon.
#  It "is a 'real-world' airline schedule planning
#  (fleet assignment) problem.  This LP was preprocessed by a modified
#  version of the KORBX(r) System preprocessor.  The problem reduced in
#  size (rows, columns, non-zeros) significantly.  The row and columns were
#  randomly sorted and renamed, and a fixed adjustment to the objective
#  function was eliminated.  The name of the problem is derived from the
#  initials of the person who created it."

#  Source:
#  The NETLIB collection of test problems.

# ----------------------------- Import libraries ----------------------------- #
import SuiteOPT
import numpy as np
import scipy

# ----------------------- Initialize problem parameters ---------------------- #
lo = np.load("Data/lo_dfl.npy")
hi = np.load("Data/hi_dfl.npy")
bl = np.load("Data/bl_dfl.npy")
bu = np.load("Data/bu_dfl.npy")
c = np.load("Data/c_dfl.npy")
A = scipy.sparse.load_npz("Data/A_dfl.npz")
nrow, ncol = A.shape 
x  = np.zeros(ncol)
parm = dict(PrintStatus=1)

# -------------------------- Initialize problem class ------------------------ #
problem = SuiteOPT.problem(ncol, nrow)
# Fill in problem parameters (bypass any parameters absent from your problem)
problem.x  = x   # Initial guess for solution
problem.lo = lo  # Lower bound on primal variable,   lo <= x
problem.hi = hi  # Upper bound on primal variable,   x <= hi
problem.A  = A   # Sparse matrix A
problem.bl = bl  # Lower bound on linear constraint, Bl <= Ax
problem.bu = bu  # Upper bound on linear constraint, Ax <= Bu
problem.c  = c   # Linear vector in quadratic cost
problem.parm = parm  # custom parameter values (optional)

# ----------------------------- Print parameters ----------------------------- #
#SuiteOPT.print_parm("all")

# -------------------------------- Call solver ------------------------------- #
newx = SuiteOPT.solve(problem)

# ------------------------------ Print solution ------------------------------ #
problem.print_stats()

if newx is not None:
    print ("SuiteOPT solution: %s" %str(newx))



# ----------------------------- Problem Details ------------------------------ #
#  NAME          HIER163A
#  A two-norm fitted formulation of the problem of finding the 
#  smallest perturbation of data that fits a linear model
#  arising in large-scale tabular data protection.

#  Source:
#  J. Castro, 
#  Minimum-distance controlled perturbation methods for 
#  large-scale tabular data protection, 
#  European Journal of Operational Research 171 (2006) pp 39-52.

# ----------------------------- Import libraries ----------------------------- #
import SuiteOPT
import numpy as np
import scipy

# ----------------------- Initialize problem parameters ---------------------- #
lo = np.load("Data/lo_qp.npy")
hi = np.load("Data/hi_qp.npy")
bl = np.load("Data/bl_qp.npy")
bu = np.load("Data/bu_qp.npy")
c = np.load("Data/c_qp.npy")
A = scipy.sparse.load_npz("Data/A_qp.npz")
H = scipy.sparse.load_npz("Data/H_qp.npz")
Hx = H.data    # Hessian matrix values
Hp = H.indptr  # Hessian matrix column pointers
Hi = H.indices # Hessian matrix row indices
nrow, ncol = A.shape 
x  = np.zeros(ncol)
parm = dict(PrintStatus=1)

# ----------------------- Wrappers for cutest functions ---------------------- #
def hprod(d,ifree):
  # Initialize m to length of vector d
  m = len(d)
  # Initialize n to length of vector ifree
  n = len(ifree)
  # Initialize output array to zeros
  Hd = np.zeros(m)
  # Check if ifree array is nonempty
  if len(ifree) != 0:
    for j in range(n):
      t = d[j]
      if t != 0:
        k = ifree[j]
        q = Hp[k+1]
        for p in range(Hp[k], q):
          Hd[ Hi[p] ] += t * Hx[p]
  else:
    for j in range(n):
      t = d[j]
      if t != 0:
        q = Hp[j+1]
        for p in range(Hp[j], q):
          Hd[ Hi[p] ] += t * Hx[p]
  return Hd

# -------------------------- Initialize problem class ------------------------ #
problem = SuiteOPT.problem(ncol, nrow)
# Fill in problem parameters (bypass any parameters absent from your problem)
problem.x  = x   # Initial guess for solution
problem.lo = lo  # Lower bound on primal variable,   lo <= x
problem.hi = hi  # Upper bound on primal variable,   x <= hi
problem.A  = A   # Sparse matrix A
problem.bl = bl  # Lower bound on linear constraint, Bl <= Ax
problem.bu = bu  # Upper bound on linear constraint, Ax <= Bu
problem.c  = c   # Linear vector in quadratic cost
problem.hprod = hprod  # Compute product hessian and vector
problem.parm = parm  # custom parameter values (optional)

# ----------------------------- Print parameters ----------------------------- #
#SuiteOPT.print_parm("all")

# -------------------------------- Call solver ------------------------------- #
newx = SuiteOPT.solve(problem)

# ------------------------------ Print solution ------------------------------ #
problem.print_stats()

if newx is not None:
    print ("SuiteOPT solution: %s" %str(newx))



# distutils: include_dirs = ../PASA/Include/ ../PPROJ/Include/ ../NAPHEAP/Include/ ../CGDESCENT/Include/ ../SuiteSparseX/SuiteSparseX_config/ ../SuiteSparseX/CHOLMOD/Include/ ../SuiteSparseX/CCOLAMD/Include/ ../SuiteSparseX/COLAMD/Include/ ../SuiteSparseX/CAMD/Include/ ../SuiteSparseX/AMD/Include/ ../SuiteSparseX/metis-5.1.0/include/ ../SuiteOPTconfig/

# ------------------------ SuiteOPT Module docstring ------------------------- #
"""
SuiteOPT: Optimization Software Suite
=====================================

SuiteOPT is designed to solve polyhedral constrained optimization
problems of the form

(1)        min                  f(x)  
       subject to    bl <= Ax <= bu, lo <= x <= hi

where f(x) is assumed to be a continuously differentiable function
and evaluation routines for f(x) and the gradient of f(x) are 
required. For Quadratic Programs, where f(x) = 0.5 * x^T H x + c^T x, 
the user can provide an evaluation routine to compute the product of 
the Hessian with a vector, H x,  and the linear term in the cost 
function. For Linear Programs, where f(x) = c^T x, the user need only 
provide the linear term c to evaluate f(x).

Additionally, SuiteOPT has routines designed to solve problems of
the form (1) with specific objective or constraint sets

Polyhedral projection problems 

(1a)       min            0.5 * || x - y ||^2  
       subject to    bl <= Ax <= bu, lo <= x <= hi

Napsack problems

(1b)       min             0.5 * x^T D x + c^T x  
       subject to    bl <= a^T x <= bu, lo <= x <= hi

where D is a diagonal matrix with diagonal entries d.

Unconstrained problems

(1c)   min f(x)  subject to  x in R^n

See Also
--------
problem : Class within SuiteOPT module for containing SuiteOPT 
          problem data

solve : Method within SuiteOPT for solving problems of the form
        (1), (1a), (1b), or (1c)
    
References
----------
W. W. Hager and H. Zhang, An Active Set Algorithm for Nonlinear
Optimization with Polyhedral Constraints, August, 2016, 
http://people.clas.ufl.edu/hager/papers/CG/pasa.pdf

W. W. Hager and H. Zhang, Sparse Techniques for Polyhedral Projection,
December 29, 2014, http://people.clas.ufl.edu/hager/

T. A. Davis, W. W. Hager, and J. T. Hungerford, An Efficient
Hybrid Algorithm for the Separable Convex Quadratic Knapsack Problem,
http://people.clas.ufl.edu/hager/

W. W. Hager and H. Zhang, A new conjugate gradient method
with guaranteed descent and an efficient line search,
SIAM Journal on Optimization, 16 (2005), 170-192.

W. W. Hager and H. Zhang, Algorithm 851: CG_DESCENT,
A conjugate gradient method with guaranteed descent,
ACM Transactions on Mathematical Software, 32 (2006), 113-137.

W. W. Hager and H. Zhang, A survey of nonlinear conjugate gradient
methods, Pacific Journal of Optimization, 2 (2006), pp. 35-58.

W. W. Hager and H. Zhang, Limited memory conjugate gradients,
SIAM Journal on Optimization, 23 (2013), 2150-2168.

Example
-------
For simplicity, a short demo for setting up and solving a problem 
of the form (1) is provided below.

import SuiteOPT
import numpy as np
# Import rosenbrock function and gradient for problem
from scipy.optimize import rosen, rosen_der

# Initialize problem data
ncol = 5
nrow = 4 
x  = -1 * np.ones(ncol)
lo = -1 * np.ones(ncol)
hi = 2 * np.ones(ncol)
A  = np.matrix([[1, 1, 1, 1, 1],[1, 1, 1, 0,-1],
                [1, 0,-1,-1, 1],[1,-1, 0, 1, 0]])
bl = np.array([3,-1,-2,-1], dtype=np.double)
bu = np.array([5, 3, 0, 1], dtype=np.double)
parm = dict(grad_tol=1e-8)

# Initialize problem class
problem = SuiteOPT.pasaProblem(ncol, nrow)

# Fill in problem data (bypass any data absent from your problem)
problem.x  = x   # Initial guess for solution
problem.lo = lo  # Lower bound on primal var,     lo <= x
problem.hi = hi  # Upper bound on primal var,     x <= hi
problem.A  = A   # Linear constr. matrix A,       bl <= Ax <= bu
problem.bl = bl  # Lower bound on linear constr., bl <= Ax
problem.bu = bu  # Upper bound on linear constr., Ax <= bu
problem.objective = rosen      # objective function
problem.gradient  = rosen_der  # gradient function
# Custom parameter values (optional)
problem.parm = parm        

# Call SuiteOPT solver and store solution in newx
newx = SuiteOPT.solve(problem)

"""

# ------------------------- Import python libraries -------------------------- #
# Import libraries for compatibility with python 3
from __future__ import print_function

# Cimport definitions from cSuiteOPT.pxd file
cimport cSuiteOPT
# Cimport numpy for array operations in C
cimport numpy as np

# Import numpy for array operations in python
import numpy as np
# Import scipy csc_matrix for sparse matrix format
from scipy.sparse import csc_matrix
# Import time for getting walltime 
import time
# Import sys for printing messages 
import sys
# Import inspect for counting number of function arguments (py 2/3 handling)
try:
    from inspect import getfullargspec as _get_args
except ImportError:
    from inspect import getargspec as _get_args

# ------------------ Import C macros ------------------ #
cdef extern from "math.h":
    float INFINITY

# ----------------------- Callback function prototypes ----------------------- #
# Define C prototype for objective function
ctypedef void (*c_objective) (double *val, double *x, int n)
# Define C prototype for gradient
ctypedef void (*c_gradient) (double *grad, double *x, int n)
# Define C prototype for objective/gradient
ctypedef void (*c_objgrad) (double *val, double *grad, double *x, int n)
# Define C prototype for hprod
ctypedef void (*c_hprod) (double *Qx, double *x, int *nzIndices, int n, int nnz)
# Define C prototype for cg_hprod
ctypedef void (*c_cg_hprod) (double *Qx, double *x, int n)

# ------------------------ Callback function wrappers ------------------------ #
# ------ C wrapper for python objective function ------ #
cdef void objective_callback (double *val, double *x, int n):
    # Set global python objective function
    global py_objective
    # Initialize output for objective function
    cdef double objective_value
    # Copy x values to variable that can be passed to python 
    cdef double [:] _x   = <double [:n]>x
    # Call python objective function
    objective_value = (<object>py_objective)(_x)
    # Store value in val
    val[0] = objective_value
    return

# ------ C wrapper for python gradient ------ #
cdef void gradient_callback (double *grad, double *x, int n):
    # Set global python gradient function
    global py_gradient
    # Copy x values to variable that can be passed to python 
    cdef double [:] _x   = <double [:n]>x
    # Initialize array to store gradient values in
    cdef double [:] gradient_values
    # Call python gradient function
    gradient_values = (<object>py_gradient)(_x)
    # Copy returned gradient_values to grad
    for i in range(n):
        grad[i] = gradient_values[i]
    return

# ------ C wrapper for python objective/gradient ------ #
cdef void objgrad_callback (double *val, double *grad, double *x, int n):
    # Set global python objective and gradient functions
    global py_objgrad
    # Initialize output for objective and gradient
    cdef double objective_value
    # Initialize array to store gradient values in
    cdef double [:] gradient_values
    # Copy x values to variable that can be passed to python 
    cdef double [:] _x   = <double [:n]>x
    # Call python objective/gradient function
    objective_value, gradient_values = (<object>py_objgrad)(_x)
    # Copy values to output variables for objective and gradient
    val[0] = objective_value
    for i in range(n):
        grad[i] = gradient_values[i]
    return

# ------ C wrapper for python hprod ------ #
cdef void hprod_callback (double *Qx, double *x, int *nzIndices, int n, int nnz):
    # Set global python objective and gradient functions
    global py_hprod
    # Initialize array to store matrix product in
    cdef double [:] matrix_product
    # Copy x values to variable that can be passed to python 
    cdef double [:] _x = <double [:n]>x
    # Copy nzIndices values to variable that can be passed to python 
    cdef int [:] _nzIndices
    if nzIndices != NULL:
        _nzIndices = <int [:nnz]>nzIndices
    else:
        _nzIndices = np.arange(n, dtype=np.int32)
    # Call python objective/gradient function
    matrix_product = (<object>py_hprod)(_x, _nzIndices)
    # Copy values to output variable
    for i in range(n):
        Qx[i] = matrix_product[i]
    return

# ------ C wrapper for python cg_hprod ------ #
cdef void cg_hprod_callback (double *Qx, double *x, int n):
    # Set global python objective and gradient functions
    global py_cg_hprod
    # Initialize array to store matrix product in
    cdef double [:] matrix_product
    # Copy x values to variable that can be passed to python 
    cdef double [:] _x = <double [:n]>x
    # Call python objective/gradient function
    matrix_product = (<object>py_cg_hprod)(_x)
    # Copy values to output variable
    for i in range(n):
        Qx[i] = matrix_product[i]
    return

# ----------------------------- problem class ---------------------------- #
class problem:
    """
    Class containing all data for problem to be solved using SuiteOPT.

    The solver in SuiteOPT is designed to solve polyhedral constrained
    problems of the form
   
    (1)        min                  f(x)  
           subject to    bl <= Ax <= bu, lo <= x <= hi

    where f(x) is assumed to be a continuously differentiable function
    and evaluation routines for f(x) and the gradient of f(x) are 
    required. For Quadratic Programs, where f(x) = 0.5 * x^T H x + c^T x, 
    the user can provide an evaluation routine to compute the product of 
    the Hessian with a vector, H x,  and the linear term in the cost 
    function. For Linear Programs, where f(x) = c^T x, the user need only 
    provide the linear term c to evaluate f(x).

    Additionally, SuiteOPT has routines designed to solve problems of
    the form (1) with specific objective or constraint sets
   
    Polyhedral projection problems 

    (1a)       min            0.5 * || x - y ||^2  
           subject to    bl <= Ax <= bu, lo <= x <= hi

    Napsack problems
    
    (1b)       min             0.5 * x^T D x + c^T x  
           subject to    bl <= a^T x <= bu, lo <= x <= hi

    where D is a diagonal matrix with diagonal entries d.

    Unconstrained problems

    (1c)   min f(x)  subject to  x in R^n


    Extended Summary
    ----------------
    Setting data after initializing 'problem' class.

    For each component required by the user's problem, the user should 
    assign the corresponding value in the 'problem' class. Note that
    vectors/matrices are expected to be a numpy array/matrix. For 
    example, if the user's problem has a lower bound constraint of
    1 for each component then the user should set

    >> problem.lo = numpy.ones(ncol)

    If the user wishes to enforce the constraint lo <= x then lo is 
    required. If the user wishes to enforce the constraint x <= hi then 
    hi is required. If lo = hi for the user's problem, then lo and hi 
    must both be provided with lo = hi.

    If the user wishes to enforce the constraint bl <= Ax <= bu then A, 
    bl, and bu are all required. Note that A can be provided in dense or 
    sparse format. If the user does not wish to enforce this constraint 
    then these values should be excluded from the problem class. If 
    bl = bu in your problem, both bl and bu must be provided as the same 
    vector.

    If the user wishes to specify the number of columns to be used from  
    matrix A, then the user should provide a value for ncol. Otherwise, 
    the value of ncol will be set to the minimum length of the following 
    elements (if provided): x, lo, hi, c, y, a, d.

    Attributes
    ----------
    ncol : int 
        Dimension of primal variable (# of columns in matrix A)
    nrow : int
        Dimension of dual variable (# of rows in matrix A)
    x : numpy array
        Initial guess for primal solution
    pylambda : numpy array 
        Initial guess for dual variable (multiplier for Ax)
    A : numpy matrix or scipy.sparse.csc_matrix
        Constraint matrix (can provide in dense or sparse format)
    bl : numpy array
        Lower bounds for Ax, bl <= Ax
    bu : numpy array
        Upper bounds for Ax, Ax <= bu
    lo : numpy array
        Lower bounds for x, lo <= x
    hi : numpy array
        Upper bounds for x, x <= hi
    c : numpy array
        Linear term in objective function (if LP or QP)
    y : numpy array 
        Point to project onto polyhedron (if solving problem (1a))
    d : numpy array
        Diagonal in cost function hessian (if solving problem (1b))
    a : numpy array
        Linear constraint vector (if solving problem (1b))
    objective : function
        Function for evaluating objective function
        Should have one input argument (point to evaluate objective)
        Should have one output argument (objective value)
    gradient : function
        Function gradient of the objective function
        Should have one input argument (point to evaluate gradient)
        Should have one output argument (gradient value)
    valgrad : function
        Function for objective and its gradient
        Should have one input argument (point to evaluate obj/grad)
        Should have two output arguments (obj & grad values)
    hprod : function
        Function for evaluating hessian times vector
    cg_hprod : 
        Function for evaluating hessian times vector in 
        unconstrained problem
    parm : dict
        Provide custom solver parameters in parm dictionary
    stats : dict
        Contains problem statistics for each method after calling 
        SuiteOPT.solve(problem)
    probtype : string
        Type of problem detected by SuiteOPT
    """

    # -------- Define __init__ function -------- #
    def __init__(self, int ncol=0, int nrow=0): 
        """
        Method for initializing instance of SuiteOPT.problem class.

        Parameters
        ----------
        ncol : Dimension of primal variable (# of columns in matrix A)
        nrow : Dimension of dual variable (# of rows in matrix A)

        Note that ncol and nrow can be ommitted and they will be 
        initialized to 0.

        Returns
        -------
        An instance of the SuiteOPT.problem class
        """

        # ------ Initialize class parameters ------ #
        # ncol must be nonnegative
        if ncol > 0:
            self.ncol = ncol
        else:
            self.ncol = 0
        # nrow must be nonnegative
        if nrow > 0:
            self.nrow = nrow
        else:
            self.nrow = 0
        self.x  = None
        self.lo = None
        self.hi = None
        self.y  = None
        self.c  = None
        self.a  = None
        self.d  = None
        self.pylambda = None
        self.bl = None
        self.bu = None
        self.A  = None
        self.parm      = dict()
        self.stats     = dict()
        self.objective = None
        self.gradient  = None 
        self.objgrad   = None 
        self.hprod     = None 
        self.cg_hprod  = None 
        self.probtype  = None
        # ------ Exit init function ------ #
        return

    # -------- Define __repr__ function -------- #
    def __repr__(self): 
        """
        Method for printing instance of SuiteOPT.problem class.

        Parameters
        ----------
        None

        Returns
        -------
        String containing all attribute names and current values
        """

        # Create dict containing all attribute names and current 
        # values in instance of problem class
        attr_dict = vars(self)
        # Return all attribute names and current values in string
        prob_repr = '<SuiteOPT.problem ' 
        prob_repr += ' '.join("%s:%s" % attr for attr in attr_dict.items())
        prob_repr += '>' 
        return prob_repr

    # -------- Define __str__ function -------- #
    def __str__(self): 
        """
        Method for printing instance of SuiteOPT.problem class.

        Parameters
        ----------
        None

        Returns
        -------
        String containing all attribute names and current values
        """

        # Create dict containing all attribute names and current 
        # values in instance of problem class
        attr_dict = vars(self)
        # Return all attribute names and current values in string
        return '\n'.join("%s: %s" % attr for attr in attr_dict.items())

    # -------- Define _print_stats function -------- #
    def print_stats(self): 
      """
      Method for printing problem statistics after solving a problem.

      After solving a problem with SuiteOPT, problem statistics are 
      stored in the SuiteOPT.problem.stats member. For each solver used 
      by SuiteOPT, there is a subdictionary within stats that contains 
      statistics for that solver, either pasa, pproj, napheap, or cg. 
      This method is used to view some key statistics generated when 
      solving a problem with SuiteOPT.

      Parameters
      ----------
      None

      Returns
      -------
      None
      """

      # Check problem type
      if self.probtype == "Unconstrained":
        # Unconstrained problem (only uses CG)
        if "cg" in self.stats:
          # Set d equal to cg statistics dictionary
          d = self.stats["cg"]
          print (_colors.OKBLUE + _colors.BOLD +
               " ===================== Problem Statistics ===================="
               + "\n" + _colors.ENDC + _colors.OKBLUE +
               "  Final objective value....................: %16.6lg\n"
               "  Final error..............................: %16.6lg\n"
               "  Walltime.................................: %16.6lg\n"
               "  Objective evals in cg....................: %16i\n"
               "  Gradient evals in cg.....................: %16i\n"
               "  Iterations in cg.........................: %16i\n"
               %(d["f"], d["err"], self.stats["walltime"], d["nfunc"], 
                 d["ngrad"], d["iter"])
               +  _colors.BOLD +
               " ============================================================="
               + _colors.ENDC)
      elif self.probtype == "Constrained":
        # Check if pasa statistics generated
        if "pasa" in self.stats:
          # Set d equal to pasa statistics dictionary
          d = self.stats["pasa"]
          # Print pasa statistics
          print (_colors.OKBLUE + _colors.BOLD +
               " ===================== Problem Statistics ===================="
               + "\n" + _colors.ENDC + _colors.OKBLUE +
               "  Final objective value....................: %16.6lg\n"
               "  Final error..............................: %16.6lg\n"
               "  Walltime.................................: %16.6lg\n"
               "  Objective evals in main code.............: %16i\n"
               "  Gradient evals in main code..............: %16i\n"
               "  Iterations in gradient projection........: %16i\n"
               "  Objective evals in gradient projection...: %16i\n"
               "  Gradient evals in gradient projection....: %16i\n"
               "  Iterations in active set grad. proj......: %16i\n"
               "  Objective evals in active set grad. proj.: %16i\n"
               "  Gradient evals in active set grad. proj..: %16i\n"
               %(d["f"], d["err"], self.stats["walltime"], d["mcnf"],  
                 d["mcng"], d["gpit"], d["gpnf"], d["gpng"], d["agpit"],  
                 d["agpnf"], d["agpng"])
               +  _colors.BOLD +
               " ============================================================="
               + _colors.ENDC)
      else:
        print (_colors.OKBLUE + _colors.BOLD +
               " ===================== Problem Statistics ===================="
               + "\n" + _colors.ENDC + _colors.OKBLUE +
               "  No problem statistics generated.\n"
               +  _colors.BOLD +
               " ============================================================="
               + _colors.ENDC)
      print ("")
      # Exit functoin
      return

# ------------------------ Check user data information ----------------------- #
# Check if user provided data with given data_name. If length of provided data
# is shorter than expected, update dimension for problem (ncol or nrow) to 
# match length of shorter data and print warning message to user
def _check_data(data_name, data_memview, dim_name, exp_len_arr):
    """Function used internally by SuiteOPT to check problem data before
       passing to solver."""
    # Check if user provided data
    if data_memview is not None:
        # Set d = length of data
        d = data_memview.shape[0]
        # User provided data; check if len matches current dimension 
        if d < exp_len_arr[0]:
            # Provided data has length smaller than current dimension
            print (_colors.BOLD + _colors.WARNING + 
                   "\n\n    PASA Warning: Length mismatch\n" + 
                   _colors.ENDC + _colors.WARNING + 
                   "      User's %s has length...: %i\n"
                   "      Expected length of %s..: %i (%s)\n" 
                   %(data_name, d, data_name, exp_len_arr[0], dim_name) +
                   "    Setting %s = %i and proceeding with PASA."
                   %(dim_name, d) + _colors.ENDC)
            # Set problem dimension to data_len
            exp_len_arr[0] = d
            return 1
        elif d == exp_len_arr[0]:
            # Provided data has length equal to current dimension
            return 0
        else:
            # Provided data has length larger than current dimension
            if exp_len_arr[0] == 0:
                # Current dimension is zero, update to length of data
                print (_colors.BOLD + _colors.WARNING + 
                       "\n\n    PASA Warning: Length mismatch\n" + 
                       _colors.ENDC + _colors.WARNING + 
                       "      User's %s has length...: %i\n"
                       "      Expected length of %s..: %i (%s)\n" 
                       %(data_name, d, data_name, exp_len_arr[0], dim_name) +
                       "    Setting %s = %i and proceeding with PASA."
                       %(dim_name, d) + _colors.ENDC)
                # Set problem dimension to data_len
                exp_len_arr[0] = d
                return 1
            else:
                # Current dimension is not zero 
                print (_colors.BOLD + _colors.WARNING + 
                       "\n\n    PASA Warning: Length mismatch\n" + 
                       _colors.ENDC + _colors.WARNING + 
                       "      User's %s has length...: %i\n"
                       "      Expected length of %s..: %i (%s)\n" 
                       %(data_name, d, data_name, exp_len_arr[0], dim_name) +
                       "    Using %s = %i and proceeding with PASA."
                       %(dim_name, exp_len_arr[0]) + _colors.ENDC)
                # Set problem dimension to data_len
                return 1
    else:
        # User did not provide data; return 0 to indicate no error
        return 0

# ----------------------- Check user matrix information ---------------------- #
# Check if user provided matrix. If dimension of provided matrix is different
# than expected, return indicator that error has occurred
def _check_matrix(A, expected_nrow, expected_ncol):
    """Function used internally by SuiteOPT to check problem matrix before
       passing to solver."""
    if (type(A) is np.matrix) or (type(A) is csc_matrix): 
        # Check if A has correct dimension (nrow x ncol)
        if A.shape != (expected_nrow, expected_ncol): 
            # A has incorrect dimensions
            print (_colors.BOLD + _colors.FAIL + 
                   "\n\n    PASA Error: Constraint matrix A has incorrect "
                   "shape."
                   + _colors.ENDC)
            print (_colors.WARNING + 
                   "      User's matrix A has shape...: %s\n"
                   "      Matrix A should have shape..: (%i, %i) "
                   "(= (nrow, ncol))\n" 
                   %(str(A.shape), expected_nrow, expected_ncol) +
                   _colors.ENDC + _colors.FAIL +
                   "    Terminating PASA.\n" + _colors.ENDC)
            print (_colors.OKBLUE +
                   "    Note: If problem does not have constraint "
                   "of the form\n"
                   "                         bl <= Ax <= bu\n"
                   "          then no value for A should be provided.\n" +
                   _colors.ENDC)
            return True
        else:
            return False
    elif A is not None: 
        # Input for A of incorrect type
        # Print error message
        print (_colors.BOLD + _colors.FAIL + 
               "\n\n    PASA Error: Constraint matrix A is of incorrect type."
               + _colors.ENDC)
        print (_colors.WARNING + 
               "      User's A is of type...: %s\n"
               "      A should be a numpy or csc matrix with shape "
               "(%i, %i),\n      which is (nrow, ncol)\n" 
               %(type(A), expected_nrow, expected_ncol) +
               _colors.ENDC + _colors.FAIL +
               "    Terminating PASA.\n" + _colors.ENDC)
        return True

# ----------------------- Check user function information -------------------- #
# Check if user provided function handle. If number of input arguments is 
# different than expected or user provided non-callable function, return 
# indicator that error has occurred
def _check_func(func_handle, func_name, expected_args):
    """Function used internally by SuiteOPT to check problem functions before
       passing to solver."""
    # Check if user provided function handle 
    if func_handle is not None:
        # Check if callable function was provided for objective
        if callable(func_handle):
            # Set func_args equal to number of input arguments
            func_args = len(_get_args(func_handle).args)
            # Check if objective has expected number of input arguments
            if func_args != expected_args:
                # Print error message
                print (_colors.BOLD + _colors.FAIL + 
                       "\n\n    PASA Error: %s function has incorrect\n"
                       "    number of input arguments.\n" %(func_name) +
                       _colors.ENDC + _colors.WARNING + 
                       "      Number of input args in user %s.....: "
                       "%i\n"
                       "      Number of input args required for %s: %i\n"
                       %(func_name, func_args, func_name, expected_args) +
                       _colors.ENDC + _colors.FAIL +
                       "    Terminating PASA.\n" + _colors.ENDC)
                # Error has occurred; return True 
                return True 
            else:
                # No errors occurred; return False 
                return False 
        else: # Non callable function provided for objective
            # Print error message
            print (_colors.BOLD + _colors.FAIL + 
                   "\n\n    PASA Error: Non-callable input provided for %s\n"
                   "    function.\n" %(func_name) +
                   _colors.ENDC + _colors.WARNING + 
                   "      %s must be callable with %i input argument(s).\n" 
                   %(func_name, expected_args) +
                   _colors.ENDC + _colors.FAIL +
                   "    Terminating PASA.\n" + _colors.ENDC)
            # Error has occurred; return True 
            return True 
    else:
        # User did not provide any input for current function; return False
        return False

# ---------------------- Print parm function definition ------------------- #
def print_parm(program_name):
    """Print all parameters associated with solvers used by SuiteOPT.
    
    When using SuiteOPT to solve a problem, the user may want to provide
    custom parameter values to the solvers used by SuiteOPT. This routine
    allows the user to check the names of all parameters used within 
    SuiteOPT and their default values.

    Parameters
    ----------
    program_name : String indicating which parameters to print

    print_parm accepts a single string as an input and returns the 
    parameter names and default values to the console. There are 4 
    different solvers that may be used by SuiteOPT when solving a 
    problem: PASA, PPROJ, NAPHEAP, and CGDESCENT. The accepted strings 
    for print_parm are provided below

    'all'     - Prints parameters for all solvers used by SuiteOPT
    'pasa'    - Prints parameters for PASA
    'pproj'   - Prints parameters for PPROJ
    'napheap' - Prints parameters for NAPHEAP
    'cg'      - Prints parameters for CGDESCENT

    Returns 
    -------
    None
    """

    # Allocate memory for pasadata and initialize parameters
    pasadata = cSuiteOPT.pasa_setup()
    # Check if user requested parameters to be printed
    if program_name == "all":
        # User requested all default parameter values
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================= Default PASA parameter values "
               "================="
               + _colors.ENDC)
        # Print pasa parameter values
        cSuiteOPT.pasa_print_parm(pasadata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n\n================ Default PPROJ parameter values "
               "================="
               + _colors.ENDC)
        # Print pproj parameter values
        cSuiteOPT.pproj_print_parm(pasadata.ppdata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n\n============== Default CG_DESCENT parameter values "
               "=============="
               + _colors.ENDC)
        # Print cg parameter values
        cSuiteOPT.cg_print_parm(pasadata.cgdata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n\n================ Default NAPHEAP parameter values "
               "==============="
               + _colors.ENDC)
        # Print napheap parameter values
        cSuiteOPT.napheap_print_parm(pasadata.napdata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================================================="
               "==============="
               + _colors.ENDC)
    elif program_name == "pasa":
        # User requested default pasa parameter values
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================= Default PASA parameter values "
               "================="
               + _colors.ENDC)
        # Print pasa parameter values
        cSuiteOPT.pasa_print_parm(pasadata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================================================="
               "==============="
               + _colors.ENDC)
    elif program_name == "pproj":
        # User requested default pproj parameter values
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================ Default PPROJ parameter values "
               "================="
               + _colors.ENDC)
        # Print pproj parameter values
        cSuiteOPT.pproj_print_parm(pasadata.ppdata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================================================="
               "==============="
               + _colors.ENDC)
    elif program_name == "cg":
        # User requested default cg parameter values
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n============== Default CG_DESCENT parameter values "
               "=============="
               + _colors.ENDC)
        # Print cg parameter values
        cSuiteOPT.cg_print_parm(pasadata.cgdata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================================================="
               "==============="
               + _colors.ENDC)
    elif program_name == "napheap":
        # User requested default napheap parameter values
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================ Default NAPHEAP parameter values "
               "==============="
               + _colors.ENDC)
        # Print napheap parameter values
        cSuiteOPT.napheap_print_parm(pasadata.napdata)
        print (_colors.BOLD + _colors.OKBLUE + 
               "\n================================================="
               "==============="
               + _colors.ENDC)
    else: # Invalid program name
        # User requested default pproj parameter values
        print (_colors.BOLD + _colors.FAIL + 
               "\nSuiteOPT error: Invalid program name provided to print_parm."
               + _colors.ENDC)
        print (_colors.BOLD + _colors.WARNING + 
               "    Functions for which parameters can be printed: pasa, "
               "pproj, cg, and napheap."
               + _colors.ENDC)
    # -------- Free memory used by pasadata -------- #
    cSuiteOPT.pasa_terminate(&pasadata)
    # ---------------- Exit program ---------------- #
    return

# --------------------------- solve function definition ----------------------- #
# Calls the pasa function to solve the given problem using either pasa, pproj,
# napheap, or cg
def solve(problem):
    """
    Method for solving problems using SuiteOPT.

    'solve' is designed to solve polyhedral constrained optimization 
    problems of the form
   
    (1)        min                  f(x)  
           subject to    bl <= Ax <= bu, lo <= x <= hi

    where f(x) is assumed to be a continuously differentiable function
    and evaluation routines for f(x) and the gradient of f(x) are 
    required. For Quadratic Programs, where f(x) = 0.5 * x^T H x + c^T x, 
    the user can provide an evaluation routine to compute the product of 
    the Hessian with a vector, H x,  and the linear term in the cost 
    function. For Linear Programs, where f(x) = c^T x, the user need only 
    provide the linear term, c, to evaluate f(x).

    Additionally, 'solve' has routines designed to solve problems of
    the form (1) with specific objective or constraint sets
   
    Polyhedral projection problems 

    (1a)       min            0.5 * || x - y ||^2  
           subject to    bl <= Ax <= bu, lo <= x <= hi

    Napsack problems
    
    (1b)       min             0.5 * x^T D x + c^T x  
           subject to    bl <= a^T x <= bu, lo <= x <= hi

    where D is a diagonal matrix with diagonal entries d.

    Unconstrained problems

    (1c)   min f(x)  subject to  x in R^n

    Parameters
    ----------
    problem : An instance of the class SuiteOPT.problem

    Returns
    -------
    xnew : Numpy array containing solution to problem

    See Also 
    --------
    problem : Class within SuiteOPT module for containing SuiteOPT 
              problem data
    """

    # -------- Initialize variables -------- #
    # Struct to provide to pasa function
    cdef cSuiteOPT.PASAdata * pasadata
    # Memview of data to provide to pasadata struct
    cdef double[::1] x_arr_memview
    cdef double[::1] lambda_arr_memview
    cdef double[::1] lo_arr_memview
    cdef double[::1] hi_arr_memview
    cdef double[::1] bl_arr_memview
    cdef double[::1] bu_arr_memview
    cdef double[::1] Ax_arr_memview
    cdef int[::1]    Ap_arr_memview
    cdef int[::1]    Ai_arr_memview
    cdef double[::1] y_arr_memview 
    cdef double[::1] c_arr_memview 
    cdef double[::1] a_arr_memview 
    cdef double[::1] d_arr_memview 
    cdef np.ndarray[np.int32_t, ndim=1] ncol_np = np.empty(1, dtype=np.int32)
    cdef np.ndarray[np.int32_t, ndim=1] nrow_np = np.empty(1, dtype=np.int32)
    # Global variables for user defined function
    global py_objective
    global py_gradient
    global py_objgrad
    global py_hprod
    global py_cg_hprod
    # Additional variables
    cdef int num_error
    cdef double t0, wall_time

    # --------------------------- PASA running message ----------------------- #
    print (_colors.OKBLUE + _colors.BOLD +
           " =============================================================\n"
           "             SuiteOPT: Optimization Software Suite\n"
           " ============================================================="
           + _colors.ENDC)

    # ------------------------ Allocate memory for C pasa -------------------- #
    print (_colors.OKBLUE + _colors.BOLD +
           "\n  Allocating memory..."
           + _colors.ENDC, end='')
    sys.stdout.flush()

    # Allocate memory for pasadata and initialize parameters
    pasadata = cSuiteOPT.pasa_setup()

    # Finished allocating memory message
    print (_colors.OKBLUE + _colors.BOLD +
           ".................................: "
           + _colors.OKGREEN + "Done"
           + _colors.ENDC)

    # ------------------------- Import custom parameters --------------------- #
    print (_colors.OKBLUE + _colors.BOLD +
           "\n  Importing custom parameters..."
           + _colors.ENDC, end='')
    sys.stdout.flush()

    # This code copies parameters from user dict to pasadata
    # Create dictionaries of parameters structs for each solver
    cdef object pasa_parms = pasadata.Parms.pasa[0]
    cdef object pproj_parms = pasadata.Parms.pproj[0]
    cdef object napheap_parms = pasadata.Parms.napheap[0]
    cdef object cg_parms = pasadata.Parms.cg[0]
    # Check if user provided correct type for parameters
    if type(problem.parm) is dict: # Import custom parameter values
        if len(problem.parm) > 0:
            # Loop over all values in user's custom parameter dictionary
            for key in problem.parm:
                # Add custom parameter to solver dictionary
                pasa_parms[key] = problem.parm[key]
                pproj_parms[key] = problem.parm[key]
                napheap_parms[key] = problem.parm[key]
                cg_parms[key] = problem.parm[key]
            # Copy imported parameter values back into each solver's 
            # parameter struct
            pasadata.Parms.pasa[0] = pasa_parms
            pasadata.Parms.pproj[0] = pproj_parms
            pasadata.Parms.napheap[0] = napheap_parms
            pasadata.Parms.cg[0] = cg_parms
            # Finished importing parameters 
            print (_colors.OKBLUE + _colors.BOLD +
                   ".......................: "
                   + _colors.OKGREEN + "Done"
                   + _colors.ENDC)
        else: # user did not provide custom parm values
            # No parameters 
            print (_colors.OKBLUE + _colors.BOLD +
                   "..............: "
                   + _colors.OKGREEN + "None provided"
                   + _colors.ENDC)
    else: # user provided non-dict for custom parm values
        # Print warning message
        print (_colors.BOLD + _colors.WARNING + 
               "\n\n    SuiteOPT Warning: Custom parameter values (parm) "
               "provided\n    as incorrect type."
               + _colors.ENDC)
        print (_colors.WARNING + 
               "      User's custom parms provided as %s.\n" 
               "      Custom parms must be provided as <type 'dict'>."
               %str(type(problem.parm))
               + _colors.ENDC)
        print (_colors.WARNING + 
               "    Proceeding with SuiteOPT using default parameter values.\n"
               + _colors.ENDC)
        print (_colors.OKBLUE + _colors.BOLD +
               "                                .......................: "
               + _colors.WARNING + "Done"
               + _colors.ENDC)

    # ---------------------------- Check user inputs ------------------------- #
    print (_colors.OKBLUE + _colors.BOLD +
           "\n  Checking problem data..."
           + _colors.ENDC, end='')
    sys.stdout.flush()

    # Note: Passing dimension data to _check_data function requires referencing address
    # Set value of ncol_np and nrow_np
    if problem.ncol is not None:
        # Check if user value for ncol is positive
        if problem.ncol > 0:
            # Set ncol equal to user value
            ncol_np[0] = problem.ncol
        else:
            # Set ncol equal to zero
            ncol_np[0] = 0
    else:
        # Set ncol equal to zero
        ncol_np[0] = 0

    if problem.nrow is not None:
        # Check if user value for nrow is positive
        if problem.nrow > 0:
            # Set nrow equal to user value
            nrow_np[0] = problem.nrow
        else:
            # Set nrow equal to zero
            nrow_np[0] = 0
    else:
        # Set nrow equal to zero
        nrow_np[0] = 0

    # Initialize memview of input data
    x_arr_memview = problem.x
    lambda_arr_memview = problem.pylambda
    lo_arr_memview = problem.lo
    hi_arr_memview = problem.hi
    bl_arr_memview = problem.bl
    bu_arr_memview = problem.bu
    y_arr_memview  = problem.y
    c_arr_memview  = problem.c
    a_arr_memview  = problem.a
    d_arr_memview  = problem.d

    # Initialize counter for number of errors during _check_data calls
    num_error = 0 

    # Check numpy arrays of expected length ncol
    num_error += _check_data("x", x_arr_memview, "ncol", ncol_np)
    num_error += _check_data("lo", lo_arr_memview, "ncol", ncol_np)
    num_error += _check_data("hi", hi_arr_memview, "ncol", ncol_np)
    num_error += _check_data("y", y_arr_memview, "ncol", ncol_np)
    num_error += _check_data("c", c_arr_memview, "ncol", ncol_np)
    num_error += _check_data("a", a_arr_memview, "ncol", ncol_np)
    num_error += _check_data("d", d_arr_memview, "ncol", ncol_np)

    # Check numpy arrays of expected length nrow
    num_error += _check_data("pylambda", lambda_arr_memview, "nrow", nrow_np)
    num_error += _check_data("bl", bl_arr_memview, "nrow", nrow_np)
    num_error += _check_data("bu", bu_arr_memview, "nrow", nrow_np)

    # Check matrix A (for shape and type) TODO update for new memview stuff
    if _check_matrix(problem.A, nrow_np[0], ncol_np[0]):
        # User has provided matrix A resulting in error; terminate program
        return

    # Check function handles (for input args and callability)
    if (_check_func(problem.objective, "objective", 1) or
        _check_func(problem.gradient, "gradient", 1) or
        _check_func(problem.objgrad, "objgrad", 1) or
        _check_func(problem.cg_hprod, "cg_hprod", 1) or
        _check_func(problem.hprod, "hprod", 2)):
        # User has provided function resulting in error; terminate program
        return

    # -------- Import problem data to pasadata -------- #
    # Set nrow and ncol based on _check_data calls
    pasadata.nrow = nrow_np[0]
    pasadata.ncol = ncol_np[0]
    if problem.x is not None:
        pasadata.x  = &x_arr_memview[0]
    if problem.lo is not None:
        pasadata.lo = &lo_arr_memview[0]
    if problem.hi is not None:
        pasadata.hi = &hi_arr_memview[0]
    if problem.y is not None:
        pasadata.y = &y_arr_memview[0]
    if problem.c is not None:
        pasadata.c = &c_arr_memview[0]
    if problem.a is not None:
        pasadata.a = &a_arr_memview[0]
    if problem.d is not None:
        pasadata.d = &d_arr_memview[0]

    if nrow_np[0] > 0:
        # TODO: lambda has set use in python, need to figure out workaround
        #pasadata.lambda  = &lambda_arr_memview[0]
        if problem.bl is not None:
            pasadata.bl = &bl_arr_memview[0]
        if problem.bu is not None:
            pasadata.bu = &bu_arr_memview[0]
        # Check user's format of matrix data
        if type(problem.A) is csc_matrix: # Sparse matrix format
            # Extract pointers to data, column, and row pointers
            Ax_arr_memview = problem.A.data
            Ap_arr_memview = problem.A.indptr
            Ai_arr_memview = problem.A.indices
            # Set pasadata structure values for matrix A
            pasadata.Ax = &Ax_arr_memview[0]
            pasadata.Ap = &Ap_arr_memview[0]
            pasadata.Ai = &Ai_arr_memview[0]
        elif type(problem.A) is np.matrix: # Dense matrix format
            # Convert numpy matrix to sparse format
            A_csc = csc_matrix(problem.A, dtype=np.double)
            # Extract pointers to data, column, and row pointers
            Ax_arr_memview = A_csc.data
            Ap_arr_memview = A_csc.indptr
            Ai_arr_memview = A_csc.indices
            # Set pasadata structure values for matrix A
            pasadata.Ax = &Ax_arr_memview[0]
            pasadata.Ap = &Ap_arr_memview[0]
            pasadata.Ai = &Ai_arr_memview[0]

    # -------- Pass function information to pasadata -------- #
    if problem.objective is not None:
        # Set py_objective to user objective function
        py_objective = problem.objective
        # Pass objective function to pasadata struct
        pasadata.value = <c_objective> objective_callback
    if problem.gradient is not None:
        # Set py_gradient to user gradient function
        py_gradient = problem.gradient
        # Pass gradient function to pasadata struct
        pasadata.grad = <c_gradient> gradient_callback
    if problem.objgrad is not None:
        # Set py_objgrad to user objgrad function
        py_objgrad = problem.objgrad
        # Pass objgrad function to pasadata struct
        pasadata.valgrad = <c_objgrad> objgrad_callback
    if problem.hprod is not None:
        # Set py_hprod to user hprod function
        py_hprod = problem.hprod
        # Pass hprod function to pasadata struct
        pasadata.hprod = <c_hprod> hprod_callback
    if problem.cg_hprod is not None:
        # Set py_cg_hprod to user cg_hprod function
        py_cg_hprod = problem.cg_hprod
        # Pass cg_hprod function to pasadata struct
        pasadata.cg_hprod = <c_cg_hprod> cg_hprod_callback

    # Indicate data checks and imports have been finished (if no errors occured)
    if num_error > 0:
        # User received warning during data check but proceeding with PASA
        print (_colors.OKBLUE + _colors.BOLD +
               "\n                          .............................: "
               + _colors.WARNING + "Done"
               + _colors.ENDC)
    else:
        # User did not receive warning during data check
        print (_colors.OKBLUE + _colors.BOLD +
               ".............................: "
               + _colors.OKGREEN + "Done"
               + _colors.ENDC)

    # ------------------- External call to C pasa function ------------------- #
    print (_colors.OKBLUE + _colors.BOLD +
           "\n  Working on solving the problem...\n\n"
           + _colors.ENDC, end='')
    sys.stdout.flush()

    # Set print status to true to ensure user gets correct status
    pasadata.Parms.pasa.PrintStatus = True
    # Start timer for pasa
    t0 = time.time()
    # Call pasa
    cSuiteOPT.pasa(pasadata)
    # Stop timer for pasa
    wall_time = time.time() - t0
    # Add walltime to statistics
    problem.stats["walltime"] = wall_time
    # Get problem status
    status = pasadata.Stats.pasa.status
    # Print indication that problem has finished
    if status == 0:
        print (_colors.OKBLUE + _colors.BOLD +
               "                                   ....................: "
               + _colors.OKGREEN + "Done\n"
               + _colors.ENDC)
    else:
        print (_colors.OKBLUE + _colors.BOLD +
               "                                   ....................: "
               + _colors.WARNING + "Done\n"
               + _colors.ENDC)

    # ------------------ Copy results/statistics and wrapup ------------------ #
    # Copy problem solution
    newx = np.zeros(ncol_np[0], dtype=np.float64)
    for i in range(ncol_np[0]):
        newx[i] = pasadata.x[i]

    # Set problem type
    if pasadata.Stats.use_pasa == 1:
      problem.probtype = "Constrained"
    elif pasadata.Stats.use_cg == 1:
      problem.probtype = "Unconstrained"

    # Copy problem statistics
    # Create dictionaries of statistics structs for each solver
    cdef object pasa_stats = pasadata.Stats.pasa[0]
    #cdef object pproj_stats = pasadata.Stats.pproj[0]
    cdef object napheap_stats = pasadata.Stats.napheap[0]
    cdef object cg_stats = pasadata.Stats.cg[0]
    # Add each statistics struct to problem.stats dictionary
    # PASA statistics
    if pasadata.Stats.use_pasa == True:
        # Add statistics to problem.stats
        problem.stats["pasa"] = pasa_stats
    else:
        # No statistics generated for pasa
        problem.stats["pasa"] = "No statistics generated"
    # NAPHEAP statistics
    if pasadata.Stats.use_napheap == True:
        # Add statistics to problem.stats
        problem.stats["napheap"] = napheap_stats
    else:
        # No statistics generated for napheap 
        problem.stats["napheap"] = "No statistics generated"
    # CG statistics
    if pasadata.Stats.use_cg == True:
        # Add statistics to problem.stats
        problem.stats["cg"] = cg_stats
    else:
        # No statistics generated for cg
        problem.stats["cg"] = "No statistics generated"

    # --------------------- Free memory used by pasadata --------------------- #
    cSuiteOPT.pasa_terminate(&pasadata)

    # ----------------------------- Exit function ---------------------------- #
    return newx


# ----------------------------- Internal classes ----------------------------- #
# ----- _colors class ----- #
class _colors:
    HEADER    = '\033[95m'
    OKBLUE    = '\033[94m'
    OKGREEN   = '\033[92m'
    WARNING   = '\033[93m'
    FAIL      = '\033[91m'
    ENDC      = '\033[0m'
    BOLD      = '\033[1m'
    UNDERLINE = '\033[4m'



# ----------------------------- Import libraries ----------------------------- #
import SuiteOPT
import numpy as np
# Import rosenbrock function and gradient for problem
from scipy.optimize import rosen, rosen_der

# ------------------------- Initialize problem data -------------------------- #
ncol = 5
nrow = 4 
x  = -1 * np.ones(ncol)
lo = -1 * np.ones(ncol)
hi = 2 * np.ones(ncol)
A  = np.matrix([[1, 1, 1, 1, 1],
                [1, 1, 1, 0,-1],
                [1, 0,-1,-1, 1],
                [1,-1, 0, 1, 0]])
bl = np.array([ 3,
               -1,
               -2,
               -1], dtype=np.double)
bu = np.array([ 5,
                3,
                0,
                1], dtype=np.double)
c  = np.zeros(ncol)
parm = dict(grad_tol=1e-8, bypass_checks=0)

# -------------------------- Initialize problem class ------------------------ #
problem = SuiteOPT.problem(ncol, nrow)
# Fill in problem parameters (bypass any parameters absent from your problem)
problem.x  = x   # Initial guess for solution
problem.lo = lo  # Lower bound on primal variable,   lo <= x
problem.hi = hi  # Upper bound on primal variable,   x <= hi
problem.A  = A   # Linear constraint matrix A,       bl <= Ax <= bu
problem.bl = bl  # Lower bound on linear constraint, bl <= Ax
problem.bu = bu  # Upper bound on linear constraint, Ax <= bu
problem.parm = parm  # custom parameter values (optional)
problem.objective = rosen      # objective function
problem.gradient  = rosen_der  # gradient function

# ----------------------------- Print parameters ----------------------------- #
#SuiteOPT.print_parm("all")

# -------------------------------- Call solver ------------------------------- #
newx = SuiteOPT.solve(problem)

# ------------------------------ Print solution ------------------------------ #
problem.print_stats()

if newx is not None:
    print ("SuiteOPT solution: %s" %str(newx))



import os
import sys
import site 
import numpy 
import subprocess 
from setuptools import find_packages
from setuptools import setup 
from setuptools.extension import Extension
from setuptools.command.build_py import build_py 
from setuptools.command.install import install as _install 
from setuptools import Command

################################################################################

# Initialize package details that do not change
PACKAGE_NAME = "SuiteOPT"
VERSION = "1.0.0"
DESCRIPTION = "Python Wrapper for Optimization Software Suite SuiteOPT"
AUTHOR = "James Diffenderfer"
AUTHOR_EMAIL = "jdiffen1@ufl.edu"
URL = "https://github.com/chrundle"
KEYWORDS = "optimization"
LICENSE = "GPLv2"
CLASSIFIERS = [ 
    "Development Status :: 4 - Beta",
    "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    "Operating System :: POSIX :: Linux",
    "Intended Audience :: Science/Research",
    "Topic :: Scientific/Engineering",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
]
# Not using install requires at the moment; causes error on some 
# systems when passing custom user BLAS and LAPACK information
INSTALL_REQUIRES = ["numpy>=1.16.x", "scipy<=1.2.9"]

# Custom class for passing user BLAS and LAPACK information during installation
class SuiteOPTInstall(_install):
  user_options = _install.user_options + [
    ('blas=', None, None), # option that takes value
    ('lapack=', None, None), # option that takes value
    ('ldlibs=', None, None), # option that takes value
    ('blfile=', None, None), # option that takes value
  ]

  def initialize_options(self):
    _install.initialize_options(self)
    self.blas = None
    self.lapack = None
    self.ldlibs = None
    self.blfile = None

  def finalize_options(self):
    # Indicate BLFILE being used
    if self.blfile != None and os.path.exists(self.blfile):
      print("User BLFILE: %s" %(self.blfile))
      # Set blas, lapack, and ldlibs from blfile
      # Copy contents of Userconfig.mk file
      with open(self.blfile, "r") as myfile:
        for line in myfile:
          # Find line with BLAS and change to user BLAS if provided
          if line.startswith("BLAS"):
            self.blas = line[4:].replace("\n","")
            # Strip leading whitespace and = signs
            self.blas = self.blas.lstrip()
            self.blas = self.blas.lstrip("=")
            self.blas = self.blas.lstrip()
          # Find line with LAPACK and change to user LAPACK if provided
          if line.startswith("LAPACK"):
            self.lapack = line[6:].replace("\n","")
            # Strip leading whitespace and = signs
            self.lapack = self.lapack.lstrip()
            self.lapack = self.lapack.lstrip("=")
            self.lapack = self.lapack.lstrip()
          # Find line with LDLIBS and change to user LDLIBS if provided
          if line.startswith("LDLIBS"):
            self.ldlibs = line[6:].replace("\n","")
            # Strip leading whitespace and = signs
            self.ldlibs = self.ldlibs.lstrip()
            self.ldlibs = self.ldlibs.lstrip("=")
            self.ldlibs = self.ldlibs.lstrip()
    else:
      print("BLFILE not provided.")

    # Indicate BLAS being used
    if self.blas != None:
      print("User BLAS: %s" %(self.blas))
    else:
      print("BLAS not provided. Using default: -lopenblas")
    # Indicate LAPACK being used
    if self.lapack != None:
      print("User LAPACK: %s" %(self.lapack))
    else:
      print("LAPACK not provided. Using default: -llapack")
    # Indicate LDLIBS being used
    if self.ldlibs != None:
      print("User LDLIBS: %s" %(self.ldlibs))
    else:
      print("LDLIBS not provided. Using default: ")

    # Get current working directory
    cwd = os.getcwd()
    # Check if SuiteOPT C pacakge is sub or parent directory of cwd or 
    # environment variable
    if os.path.isdir(cwd + "/SuiteOPT"): 
      # SuiteOPT C package is subdirectory of cwd
      SUITEOPTDIR = cwd + "/SuiteOPT"
    elif os.path.isdir(cwd + "/../../SuiteOPT"): 
      # SuiteOPT C package is parent directory of cwd
      SUITEOPTDIR = os.path.abspath(os.path.join(cwd,os.pardir))

    # Copy contents of Userconfig.mk file
    with open(SUITEOPTDIR + "/SuiteOPTconfig/Userconfig.mk", "r") as myfile:
      # Save contents of Userconfig.mk to tmpdata
      tmpdata = myfile.readlines()
    # Set ndata equal to length of tmpdata
    ndata = len(tmpdata)
    # Find line with BLAS and change to user BLAS if provided
    if self.blas != None:
      for i in range(ndata-1,0,-1):
        if tmpdata[i].startswith("BLAS"):
          tmpdata[i] = "BLAS = " + self.blas + "\n"
          break
      # If MKL blas not user need to add DNSUPER flag to OPTFLAGS
      for i in range(ndata-1,0,-1):
        if tmpdata[i].startswith("OPTFLAGS +="):
          if "mkl" in self.blas:
            tmpdata[i] = "OPTFLAGS += \n"
          else:
            tmpdata[i] = "OPTFLAGS += -DNSUPER\n"
          break
    # Find line with LAPACK and change to user LAPACK if provided
    if self.lapack != None:
      for i in range(ndata-1,0,-1):
        if tmpdata[i].startswith("LAPACK"):
          tmpdata[i] = "LAPACK = " + self.lapack + "\n"
          break
    # Find line with LDLIBS and change to user LDLIBS if provided
    if self.ldlibs != None:
      for i in range(ndata-1,0,-1):
        if tmpdata[i].startswith("LDLIBS"):
          tmpdata[i] = "LDLIBS = " + self.ldlibs + "\n"
          break
    # Print updated data back to file
    with open(SUITEOPTDIR + "/SuiteOPTconfig/Userconfig.mk", "w") as myfile:
      # Save modified contents of tmpdata back to Userconfig.mk
      myfile.writelines(tmpdata)

    # Call standard finalize_options for install
    _install.finalize_options(self)

  def run(self):
    global blas, lapack, ldlibs, blfile
    blas = self.blas
    lapack = self.lapack
    ldlibs = self.ldlibs
    blfile = self.blfile

    # This attempts to solve problem of libpasa.so and pibpproj.so being
    # unable to locate SuiteSparse dynamic libraries at runtime from 
    # within python
    # Set environment variable for linking SuiteSparse libraries at runtime
    # Note: This environment variable is appended to LDLIBS in compilation
    # of dynamic libraries for SuiteOPT and SuiteSparse
    # ---- Set directories to search for dynamic libraries at runtime ---- #
    # Initialize list to empty list
    runtime_library_dirs = []
    # Add SuiteOPT and SuiteSparse dynamic library directories to all 
    # directories from getusersitepackages to list (typically for site-packages)
    # as these directories need to be searched for by SuiteSparse and SuiteOPT
    # dynamic libraries at runtime
    usersitepackages = site.getusersitepackages()
    if type(usersitepackages) == str:
      runtime_library_dirs += [usersitepackages + "/SuiteOPT-libraries"]
      runtime_library_dirs += [usersitepackages + "/SuiteSparse-libraries"]
    elif type(usersitepackages) == list:
      runtime_library_dirs += [x + "/SuiteOPT-libraries" for x in usersitepackages]
      runtime_library_dirs += [x + "/SuiteSparse-libraries" for x in usersitepackages]
    # Add SuiteOPT and SuiteSparse dynamic library directories to all 
    # directories from getsitepackages to list (typically for dist-packages)
    # as these directories need to be searched for by SuiteSparse and SuiteOPT
    # dynamic libraries at runtime
    userdistpackages = site.getsitepackages()
    if type(userdistpackages) == str:
      runtime_library_dirs += [userdistpackages + "/SuiteOPT-libraries"]
      runtime_library_dirs += [userdistpackages + "/SuiteSparse-libraries"]
    elif type(userdistpackages) == list:
      runtime_library_dirs += [x + "/SuiteOPT-libraries" for x in userdistpackages]
      runtime_library_dirs += [x + "/SuiteSparse-libraries" for x in userdistpackages]
    # SuiteOPT will be looking for BLAS and LAPACK libraries at runtime
    if self.ldlibs != None:
      # Append list of ldlibs provided by user after stripping -L
      runtime_library_dirs += [x[2:] for x in self.ldlibs.split()]
    # For all directories in runtime_library_dirs, prepend "-Wl,-rpath," so
    # so that compiler knows to search these directories at runtime
    runtime_library_dirs = ["-Wl,-rpath," + x for x in runtime_library_dirs]
    # Join all flags and directories into single string
    runtime_library_dirs = " ".join(runtime_library_dirs)
    os.putenv("PYSUITEOPT_RPATH", runtime_library_dirs)
    #os.putenv("SUITEOPT_RPATH", "-Wl,-rpath," + site.getusersitepackages() + "/SuiteSparse-libraries")
    # Make SuiteSparse and SuiteOPT libraries
    suiteopt_command = ["make"]
    # Check if errors occurred during make command
    if subprocess.call(suiteopt_command) != 0:
      sys.exit(-1)
    # Update LDFLAGS environment variable to include user LDLIBS 
    # This is required since the extension will be looking for these
    # at time of compilation of SuiteOPT.so
    if self.ldlibs != None:
      ldflags = self.ldlibs + " " + self.blas + " " + self.lapack + " "
      try:
        os.environ["LDFLAGS"] += ldflags
      except:
        os.environ["LDFLAGS"] = ldflags
    # Execute regular install commands
    _install.run(self)

# ---- Update information required for setup on user's system ---- #
# Initialize package details that are later updated
setup_requires = ["numpy>=1.16.x", "setuptools", "scipy"]

# Get current working directory
cwd = os.getcwd()
# Check if SuiteOPT C pacakge is sub or parent directory of cwd or 
# environment variable
if os.path.isdir(cwd + "/SuiteOPT"): 
  # SuiteOPT C package is subdirectory of cwd
  SUITEOPTDIR = "SuiteOPT/"
elif os.path.isdir(cwd + "/../../SuiteOPT"): 
  # SuiteOPT C package is parent directory of cwd
  SUITEOPTDIR = "../"

# ---- Set directories to search for dynamic libraries at compilation ---- #
# Initialize library_dirs list to be empty
library_dirs = []
# Append all required library directories to library_dirs
library_dirs.append(SUITEOPTDIR + "lib")
library_dirs.append(SUITEOPTDIR + "SuiteSparseX/lib")
#library_dirs += BLASDIRS

# ---- Set directories to search for dynamic libraries at runtime ---- #
# Initialize list to empty list
runtime_library_dirs = []
# Add SuiteOPT and SuiteSparse dynamic library directories to all 
# directories from getusersitepackages to list (typically for site-packages)
usersitepackages = site.getusersitepackages()
if type(usersitepackages) == str:
  runtime_library_dirs += [usersitepackages + "/SuiteOPT-libraries"]
  runtime_library_dirs += [usersitepackages + "/SuiteSparse-libraries"]
elif type(usersitepackages) == list:
  runtime_library_dirs += [x + "/SuiteOPT-libraries" for x in usersitepackages]
  runtime_library_dirs += [x + "/SuiteSparse-libraries" for x in usersitepackages]
# Add SuiteOPT and SuiteSparse dynamic library directories to all 
# directories from getsitepackages to list (typically for dist-packages)
userdistpackages = site.getsitepackages()
if type(userdistpackages) == str:
  runtime_library_dirs += [userdistpackages + "/SuiteOPT-libraries"]
  runtime_library_dirs += [userdistpackages + "/SuiteSparse-libraries"]
elif type(userdistpackages) == list:
  runtime_library_dirs += [x + "/SuiteOPT-libraries" for x in userdistpackages]
  runtime_library_dirs += [x + "/SuiteSparse-libraries" for x in userdistpackages]

# ---- Set extra link arguments to add directories to search at runtime ---- #
# For some reason adding the runtime_library_dirs only works for SuiteOPT
# dynamic libraries and not SuiteSparse, trying workaround
# Initialize extra_link_args
extra_link_args = []
# Append all runtime_library_dirs with -Wl and -rpath flags
for d in runtime_library_dirs:
  extra_link_args.append("-Wl,-rpath," + d) 

# ---- Set directories to search for header files at compilation ---- #
# Initialize include_dirs list to contain numpy.get_include()
include_dirs = [numpy.get_include()]
# Append all required include directories to include_dirs
include_dirs.append(SUITEOPTDIR + "include")
include_dirs.append(SUITEOPTDIR + "SuiteSparseX/include")

# ---- Set list of libraries required at compilation ---- #
# Initialize libraries list to contain required SuiteOPT and SuiteSparse libs
libraries = ["pasa", "pproj", "cg_descent", "napheap", "cholmod",
             "ccolamd", "colamd", "camd", "amd", "metis"]
# Add all additional user libraries to libraries list
#libraries += USERLIBS

# ---- Set extra compiler arguments ---- #
extra_compile_args = ["-std=c99", "-lm", "-lpthread", "-fopenmp", "-Wno-cpp", "-O3"]

# ---- Set custom command class (custom install routine so user can provide BLAS/LAPACK) ---- #
# Set cmdclass arguments
cmdclass = {}
# Customize install cmdclass to allow user to pass custom BLAS and LAPACK
cmdclass['install'] = SuiteOPTInstall

# ---- Set packages to contain dynamic libraries from SuiteSparse and SuiteOPT ---- #
# Initialize pacakges to empty list
packages = []
# Initialize package_dir as empty dicionary
package_dir = {}
# Initialize package_data as empty dicionary
package_data = {}

# ---- Add SuiteOPT dynamic library data to packages ---- #
# Add SuiteOPT-libraries to list of packages
packages.append("SuiteOPT-libraries")
# Add SuiteOPT/lib to SuiteOPT-libraries package directories in dictionary
package_dir["SuiteOPT-libraries"] = SUITEOPTDIR + "lib"
# Add SuiteOPT dynamic libraries to SuiteOPT-libraries package data in dictionary
package_data["SuiteOPT-libraries"] = ["*.so*"]

# ---- Add SuiteSparse dynamic library data to packages ---- #
# Add SuiteSparse-libraries to list of packages
packages.append("SuiteSparse-libraries")
# Add SuiteOPT/SuiteSparseX/lib to SuiteSparse-libraries package directories in dictionary
package_dir["SuiteSparse-libraries"] = SUITEOPTDIR + "SuiteSparseX/lib"
# Add SuiteSparse dynamic libraries to SuiteSparse-libraries package data in dictionary
package_data["SuiteSparse-libraries"] = ["*.so*"]

# USE_CYTHON should be True to rebuild SuiteOPT.c file using cython
# USE_CYTHON should be False to build SuiteOPT.so library from prebuilt SuiteOPT.c file (for sdist)
USE_CYTHON = False

# Set extensions, setup_requires, and run setup
if USE_CYTHON:
  # Using cython
  from Cython.Build import cythonize 
  # Setup requires
  setup_requires += ["Cython>=0.29.x"]
  # Include path specifies where the SuiteOPT.pxd file is located and
  # is a cythonize option (not an Extension option)
  include_path = ["pySuiteOPT"]
  # Set extensions
  extensions = cythonize([Extension("SuiteOPT",
                                    ["pySuiteOPT/SuiteOPT.pyx"],
                                    include_dirs = include_dirs,
                                    libraries = libraries,
                                    library_dirs = library_dirs,
                                    extra_link_args = extra_link_args,
                                    extra_compile_args = extra_compile_args)],
                         include_path = include_path)
  # Setup
  setup (
    name = PACKAGE_NAME,
    version = VERSION,
    description = DESCRIPTION,
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = URL,
    keywords = KEYWORDS,
    license = LICENSE,
    classifiers = CLASSIFIERS,
    cmdclass = cmdclass,
    packages = packages,
    package_dir = package_dir,
    package_data = package_data,
    include_package_data = True,
    ext_modules = extensions
  )
else:
  # Not using cython
  extensions = [Extension("SuiteOPT", ["pySuiteOPT/SuiteOPT.c"], 
                          include_dirs = include_dirs,
                          libraries = libraries,
                          library_dirs = library_dirs,
                          extra_link_args = extra_link_args,
                          extra_compile_args = extra_compile_args)]
  # Setup
  setup (
    name = PACKAGE_NAME,
    version = VERSION,
    description = DESCRIPTION,
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = URL,
    keywords = KEYWORDS,
    license = LICENSE,
    classifiers = CLASSIFIERS,
    cmdclass = cmdclass,
    packages = packages,
    package_dir = package_dir,
    package_data = package_data,
    include_package_data = True,
    ext_modules = extensions
  )
